package dominio;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.print.Book;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Calendar;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.filechooser.FileSystemView;
import javax.swing.table.DefaultTableModel;

import persistencia.Persistencia;
import presentacion.IVentana;

import com.jrefinery.report.ElementAlignment;
import com.jrefinery.report.ItemFactory;
import com.jrefinery.report.JFreeReport;
import com.jrefinery.report.ReportProcessingException;
import com.jrefinery.report.ShapeElement;
import com.jrefinery.report.TextElement;
import com.jrefinery.report.preview.PreviewDialog;
import com.jrefinery.report.targets.style.ElementStyleSheet;

import configuracion.Configuracion;

import dominio.exceptions.CitaConFechaNoAsignadaException;
import dominio.exceptions.CitaNoExisteException;
import dominio.exceptions.CitaYaAsigndaException;
import dominio.exceptions.DemasiadasCitasException;
import dominio.exceptions.ListaNoCambiadaException;
import dominio.exceptions.ListasException;
import dominio.reportes.ReportePrinter;

public class Control
{


	private Hashtable<String, Mes> meses = null;

	private IVentana v;

	private Configuracion c;



	public Control(IVentana v){
		this.v = v;

		if(Persistencia.existeConfiguracion()){
			try {

				c = Persistencia.getConfiguration();

			} catch (Exception e1) {
				v.showError(e1);
				c = new Configuracion();
			}

		}else{
			c = new Configuracion();
		}

		if(c.getPathBaseCitas() != null){
			File f = new File(c.getPathBaseCitas());
			try {
				meses = Persistencia.readDataBase(f);
			}  catch (Exception e) {
				v.showError(new Exception("No se ha podido leer la base datos. Compruebe que hay acceso al archivo donde est almacenada la nueva base de datos. Si el problema persiste puede que el archivo de base de datos est daado. Intente utilizar la copia de seguridad."));

				JFileChooser fc = new JFileChooser("Seleccione el archivo que contiene la copia de seguridad (el cual suele estar almacenado en la misma carpeta que el archivo regular)");
				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				FileNameExtensionFilter filter = new FileNameExtensionFilter("Archivos de tipo CitasMed", "citasMed_Security_Copy");
				fc.setFileFilter(filter);
				int seleccion = fc.showOpenDialog(null);

				if(seleccion == JFileChooser.APPROVE_OPTION){
					File fichero = fc.getSelectedFile();
					try {
						meses = Persistencia.readDataBase(fichero);
					} catch (Exception e1) {
						v.showError(new Exception("No se ha podido leer la copia de seguridad."));

					}
				}
			}

		}else{
			this.v.showMensaje("No existe ninguna base de datos asociada al sistema. Por favor, crea una nueva.");


			boolean continuar = true;
			while(continuar){

				JFileChooser fc = new JFileChooser("De un nombre a la nueva base de datos");
				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				FileNameExtensionFilter filter = new FileNameExtensionFilter("Archivos de tipo CitasMed", "citasMed");
				fc.setFileFilter(filter);
				int seleccion = fc.showSaveDialog(null);
				if(seleccion == JFileChooser.APPROVE_OPTION){
					File fichero = fc.getSelectedFile();
					if(!fichero.getAbsolutePath().endsWith(".citasMed")){
						fichero = new File(fichero.getAbsolutePath()+".citasMed");
					}
					try {
						this.nuevaDataBaseCero(fichero);
						continuar = false;
					} catch (Exception e) {
						e.printStackTrace();
					}
				}else{
					continuar = false;
				}

			}
		}
		if (meses == null){
			System.exit(0);
		}

		this.borrarExtraData();
	}


	private void borrarExtraData(){

		int mTS = this.c.getMesesToStore();

		int aosTS = mTS/12;
		int mesesTS = mTS%12;

		int actualMes = Calendar.getInstance().get(Calendar.MONTH)+1;
		int actualAo = Calendar.getInstance().get(Calendar.YEAR);

		int finalmes;
		int finalao;

		if(actualMes>mesesTS){
			finalmes = (actualMes-mesesTS)-1;
			finalao = actualAo-aosTS;
		}else{
			finalmes = (12 - (mesesTS-actualMes))-1;
			finalao = actualAo-aosTS-1;
		}


		Vector<String> keys = new Vector<String>();
		keys.addAll(this.meses.keySet());
		for(int i = 0;i<keys.size();i++){
			String key = keys.get(i);
			int ao = Integer.parseInt(key.substring(0, 4));
			String mes = key.substring(4);
			int mesI=0;

			if(mes.equals("Enero")){
				mesI=0;
			}else if(mes.equals("Febrero")){
				mesI=1;
			}else if(mes.equals("Marzo")){
				mesI=2;
			}else if(mes.equals("Abril")){
				mesI=3;
			}else if(mes.equals("Mayo")){
				mesI=4;
			}else if(mes.equals("Junio")){
				mesI=5;
			}else if(mes.equals("Julio")){
				mesI=6;
			}else if(mes.equals("Agosto")){
				mesI=7;
			}else if(mes.equals("Septiembre")){
				mesI=8;
			}else if(mes.equals("Octubre")){
				mesI=9;
			}else if(mes.equals("Noviembre")){
				mesI=10;
			}else if(mes.equals("Diciembre")){
				mesI=11;
			}

			if(ao < finalao || (ao == finalao && mesI<finalmes)){
				this.meses.remove(key);
			}

		}

	}

	public void cerrarApplicacion(){

		try {
			Persistencia.saveConfiguration(c);
		} catch (Exception e) {
			this.v.showError(e);
		}

	}


	/***********Metodos para la configuracin
	 *
	 *********************/
	public  String getPathDBCitas(){

		return c.getPathBaseCitas();

	}


	public  int getMesesToStore(){

		return c.getMesesToStore();
	}

	public  void setConfiguration(int m){
		this.c.setMesesToStore(m);

	}


	/***************Metodos para guardar archivos******
	 *
	 *************/

	public void nuevaDataBase(File f) throws Exception{
		if(this.getPathDBCitas()!=null){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				v.showError(new Exception("No se ha podido guardar la base de datos actual. No se ha creado la nueva base de datos.\nCompruebe que hay acceso al disco donde est almacenada la base de datos y que el archivo existe. Si el problema persiste intente guardar la base de datos con otro nombre con la opcin Guardar Como"));
				throw e;
			}
		}


		String pathOld = this.c.getPathBaseCitas();
		this.c.setPathBaseCitas(f.getAbsolutePath());
		this.meses = new Hashtable<String, Mes>();
		try {
			this.guardarDataBase();
		} catch (Exception e) {
			v.showError(new Exception("No se ha podido guardar la nueva base datos. Compruebe que hay acceso al disco donde se quiere almacenada la nueva base de datos. Si el problema persiste intente guardar la nueva base de datos en otra ubicacin"));
			this.c.setPathBaseCitas(pathOld);
			throw e;
		}


		this.v.refresh();
	}

	public void nuevaDataBaseCero(File f) throws Exception{

		String pathOld = this.c.getPathBaseCitas();
		this.c.setPathBaseCitas(f.getAbsolutePath());
		this.meses = new Hashtable<String, Mes>();
		try {
			this.guardarDataBaseCero();
		} catch (Exception e) {
			v.showError(new Exception("No se ha podido guardar la nueva base datos. Compruebe que hay acceso al disco donde se quiere almacenada la nueva base de datos. Si el problema persiste intente guardar la nueva base de datos en otra ubicacin"));
			this.c.setPathBaseCitas(pathOld);
			throw e;
		}
	}

	/*public void guardarComo(File f) throws Exception{


		String pathOld = this.c.getPathBaseCitas();
		this.c.setPathBaseCitas(f.getAbsolutePath());
		this.meses = new Hashtable<String, Mes>();
		try {
			this.guardarDataBase();
		} catch (Exception e) {
			v.showError(new Exception("No se ha podido guardar la base datos en la nueva ubicacin. Compruebe que hay acceso al disco donde se quiere almacenada la base de datos. Si el problema persiste intente guardar la base de datos en otra ubicacin"));
			this.c.setPathBaseCitas(pathOld);
			throw e;
		}


		this.v.refresh();
	}*/

	public void cambiarDataBase(File f)throws Exception{
		if(this.getPathDBCitas()!=null){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				v.showError(new Exception("No se ha podido guardar la base de datos actual. No se ha creado la nueva base de datos.\nCompruebe que hay acceso al disco donde est almacenada la base de datos y que el archivo existe. Si el problema persiste intente guardar la base de datos con otro nombre con la opcin Guardar Como"));
				throw e;
			}
		}

		try {
			this.meses = this.leerDataBase(f);
			this.borrarExtraData();
		} catch (Exception e) {
			v.showError(new Exception("No se ha podido leer la nueva base datos. Compruebe que hay acceso al disco donde se quiere almacenada la nueva base de datos. Si el problema persiste puede que el archivo de base de datos est daado y no se pueda utilizar"));
			throw e;
		}
		this.c.setPathBaseCitas(f.getAbsolutePath());


		this.v.refresh();
	}

	/*public  void guardarComoDataBase(File f) throws IOException{
		this.c.setPathBaseCitas(f.getAbsolutePath());
		Persistencia.saveDataBase(f, meses);

	}*/

	public  void guardarDataBase() throws IOException{
		File f = new File(c.getPathBaseCitas());
		this.guardarCopiaDeSeguridad();
		Persistencia.saveDataBase(f, meses);
	}

	public  void guardarDataBaseCero() throws IOException{
		File f = new File(c.getPathBaseCitas());
		Persistencia.saveDataBase(f, meses);
	}

	public void guardarCopiaDeSeguridad() throws IOException{
		File f = new File(c.getPathBaseCitas()+"_Security_Copy");
		Persistencia.saveDataBase(f, meses);

	}

	/*public void borrarCopiaDeSeguridad(){
		File f = new File(c.getPathBaseCitas()+"_Security_Copy");
		Persistencia.borrarDataBase(f);
	}*/



	public Hashtable<String, Mes> leerDataBase(File f) throws IOException, ClassNotFoundException{
		return Persistencia.readDataBase(f);

	}



	/*************Manejar citas*************
	 *
	 ****************************/



	public boolean addCitaSinFecha(int nh, String nombre, String comment,int preferencia, int ao, String mes){

		boolean aadida = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		Cita c = new Cita(nombre, nh, comment, preferencia);
		try {
			m.asignarCitaSinFecha(c);
			aadida = true;
		} catch (CitaYaAsigndaException e) {
			v.showError(e);
		}

		if(aadida){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return aadida;
	}

	public boolean addCitaConFecha(int nh, String nombre, String comment,int preferencia, int ao, String mes, int dia){
		boolean aadida = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		Cita c = new Cita(nombre, nh, comment, preferencia);
		try {
			m.asignarCitaConFecha(c, dia, false);
			aadida = true;
		} catch (CitaYaAsigndaException e) {
			v.showError(e);
		} catch (CitaConFechaNoAsignadaException e) {
			v.showError(e);
		}

		if(aadida){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return aadida;
	}

	public boolean addCitaEspecial(int nh, String nombre, String comment,int preferencia, int ao, String mes, int dia){
		boolean aadida = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		Cita c = new Cita(nombre, nh, comment, preferencia);
		try {
			m.asignarCitaConFecha(c, dia, true);
			aadida = true;
		} catch (CitaYaAsigndaException e) {
			v.showError(e);
		} catch (CitaConFechaNoAsignadaException e) {
			v.showError(e);
		}

		if(aadida){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return aadida;
	}


	public boolean asignarDiaCita(int nh, int ao, String mes, int dia){
		boolean asignada = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		try {
			m.asignarDaACitaSinDa(nh, dia);
			asignada = true;
		} catch (CitaNoExisteException e) {
			v.showError(e);
		} catch (CitaConFechaNoAsignadaException e) {
			v.showError(e);
		} catch (CitaYaAsigndaException e) {
			v.showError(e);
		}

		if(asignada){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return asignada;
	}


	/**
	 * listas data es un vector de pares de enteros. cada par representa una lista
	 * El primer nmeor del par indica el da del mes.
	 * El segundo nmero indica el nmero mximo de pacientes que se pueden citar.
	 * @param ao
	 * @param mes
	 * @param listasData
	 * @return
	 */
	public boolean crearListas(int ao, String mes, Vector<Integer[]> listasData){
		boolean creadas = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		for(int i = 0; i<listasData.size();i++){
			Integer [] pd =  listasData.get(i);
			if(!m.getListas().containsKey(pd[0])){
				m.crearLista(pd[0], pd[1]);
			}else{
				listasData.remove(i);
				i--;
			}

		}

		try {
			m.organizarAsignados();
			creadas = true;
		} catch (DemasiadasCitasException e) {
			for(Integer [] pd: listasData){
				m.borrarLista(pd[0]);

			}
			v.showError(e);
		}

		if(creadas){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return creadas;
	}

	public boolean cambiarListaDeDa(int ao, String mes, int diaO, int diaF){
		boolean cambiada = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);

			ListasException le = new ListasException(mes+" de "+ao+" no tiene listas creadas, por lo que no se pueden cambiar de da.");
			v.showError(le);
			return cambiada;
		}

		try {
			String men = m.cambiarListaDeDia(diaO, diaF);
			v.showMensaje(men);
			cambiada = true;
		} catch (ListaNoCambiadaException e) {
			v.showError(e);
		}

		if(cambiada){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return cambiada;
	}

	public boolean cambiarMaxLista(int ao, String mes, int dia, int newMax){
		boolean cambiado = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);

			ListasException le = new ListasException(mes+" de "+ao+" no tiene listas creadas, por lo que no se pueden cambiar el mximo de citas.");
			v.showError(le);
			return cambiado;
		}

		try {
			m.cambiarMaxLista(dia, newMax);
			cambiado = true;
		} catch (ListaNoCambiadaException e) {
			v.showError(e);
		}

		if(cambiado){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return cambiado;
	}

	public boolean borrarLista(int ao, String mes, int dia){
		boolean borrado = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);

			ListasException le = new ListasException(mes+" de "+ao+" no tiene listas creadas, por lo que no se pueden borrar.");
			v.showError(le);
			return borrado;
		}

		borrado = m.borrarLista(dia);
		if(!borrado){
			v.showError(new ListasException("No existe una lista el "+dia+" de "+mes+" de "+ao));
		}

		if(borrado){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return borrado;
	}




	public boolean oganizarAsignados(int ao, String mes){
		boolean organizados = false;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		try {
			m.organizarAsignados();
			organizados = true;
		} catch (DemasiadasCitasException e) {
			v.showError(e);
		}

		if(organizados){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return organizados;
	}




	public Vector<ResultadoBusqueda> buscarCitasPorNombre(String nombre){
		Vector<ResultadoBusqueda> retorno = new Vector<ResultadoBusqueda>();

		for(Mes m: this.meses.values()){
			retorno.addAll(m.buscarCitaPorNombre(nombre));
		}

		return retorno;
	}

	public Vector<ResultadoBusqueda> buscarCitasPorNH(int nh){
		Vector<ResultadoBusqueda> retorno = new Vector<ResultadoBusqueda>();

		for(Mes m: this.meses.values()){
			retorno.addAll(m.buscarCitaPorNH(nh));
		}

		return retorno;
	}


	public boolean borrarCita(String mes, int ao, int nh,boolean especial, int dia){
		boolean borrado = true;
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		Cita c = m.borrarCita(nh,especial,dia);

		if(c == null){
			borrado =false;
		}

		if(borrado){
			try {
				this.guardarDataBase();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return borrado;
	}

	public Mes getMes(String mes, int ao) {
		// TODO Auto-generated method stub
		String idMes = ao+mes;
		Mes m = meses.get(idMes);
		if(m==null){
			m = new Mes(mes, ao);
			meses.put(idMes, m);
		}

		return m;
	}



	/***********Metodos para la impresin
	 *
	 ************************************/

	public static final int CITAS_NORMALES = 0;
	public static final int CITAS_ESPECIALES = 1;
	public static final int CITAS_TODAS = 2;

	public boolean imprimirMes(String mes, int ao, int citasOption, boolean printComments, PageFormat pf){
		ReportePrinter p = new ReportePrinter(this);
		return p.imprimirMes(mes, ao, citasOption, printComments, pf, null);
	}

	public boolean imprimirDia(String mes, int ao, int citasOption, boolean printComments, PageFormat pf, Lista l){
		ReportePrinter p = new ReportePrinter(this);
		return p.imprimirMes(mes, ao, citasOption, printComments, pf, l);
	}










}