package dominio;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.TreeSet;
import java.util.Vector;

import dominio.exceptions.CitaConFechaNoAsignadaException;
import dominio.exceptions.CitaNoExisteException;
import dominio.exceptions.CitaYaAsigndaException;
import dominio.exceptions.DemasiadasCitasException;
import dominio.exceptions.ListaNoCambiadaException;

public class Mes implements Serializable{

	 /**
	 *
	 */
	private static final long serialVersionUID = 3L;
	private Hashtable<Integer, Cita> sinAsignarDa;
	 private Hashtable<Integer, Cita> sinAsignarDaEspecial;


	 String mes;
	 int ao;

	 private Hashtable<Integer, Lista> listas;

	 public Mes(String mes, int ao){

		 this.mes = mes;
		 this.ao = ao;
		 sinAsignarDa = new Hashtable<Integer, Cita>();
		 sinAsignarDaEspecial = new Hashtable<Integer, Cita>();
		 listas =new Hashtable<Integer, Lista>();

	 }

	 public void asignarCitaSinFecha(Cita c) throws CitaYaAsigndaException{
		 if(this.sinAsignarDa.containsKey(c.getNumeroHistoria())){
			 throw new CitaYaAsigndaException("Ya se ha asignado un paciente con el nmero de historia "+c.getNumeroHistoria()+" este mes. Intente asignar la nueva cita con un da concreto para este mes o de forma especial.");
		 }

		 this.sinAsignarDa.put(c.getNumeroHistoria(),c);
	 }

	 public void asignarCitaConFecha(Cita c, int dia, boolean especial) throws CitaConFechaNoAsignadaException, CitaYaAsigndaException{

		 if(this.listas.size()==0){
			 throw new CitaConFechaNoAsignadaException("No hay listas asignadas a "+this.mes+" de "+this.ao);
		 }


		 Lista l = this.listas.get(dia);

		 if(l == null){
			 throw new CitaConFechaNoAsignadaException("No hay ninguna lista asignda al "+dia+" de "+this.mes+" de "+this.ao);
		 }else{
			 if(!especial){
				 l.addAsignado(c);
			 }else{
				 l.addEspecial(c);
			 }
		 }
	 }


	 public Cita borrarCita(int nh, boolean especial, int dia){
		 Cita c = null;
		 if(dia == -1){
			 if(especial){
				c = this.sinAsignarDaEspecial.remove(nh);
			 }else{
				c = this.sinAsignarDa.remove(nh);
			 }
		 }else{
			 for(Lista l: this.listas.values()){
				if(l.getDia() == dia){
					if(especial){
						c = l.borrarEspecial(nh);
					}else{
						c = l.borrarAsignado(nh);
					}
				}
			 }
		 }
		 return c;
	 }

	 public Vector<ResultadoBusqueda> buscarCitaPorNombre(String nombre){

		 Vector<ResultadoBusqueda> citas = new Vector<ResultadoBusqueda>();

		 for(Cita c:this.sinAsignarDa.values()){
			 if( c.getNombre().contains(nombre)){
				 ResultadoBusqueda rb = new ResultadoBusqueda(c, null, this, false);
				 citas.add(rb);
			 }
		 }

		 for(Cita c:this.sinAsignarDaEspecial.values()){
			 if( c.getNombre().contains(nombre)){
				 ResultadoBusqueda rb = new ResultadoBusqueda(c, null, this, true);
				 citas.add(rb);
			 }
		 }


		 for(Lista l:this.listas.values()){
			 for(Cita c:l.getAsignados().values()){
				 if( c.getNombre().contains(nombre)){
					 ResultadoBusqueda rb = new ResultadoBusqueda(c, l, this, false);
					 citas.add(rb);
				 }
			 }
			 for(Cita c:l.getEspeciales().values()){
				 if( c.getNombre().contains(nombre)){
					 ResultadoBusqueda rb = new ResultadoBusqueda(c, l, this, true);
					 citas.add(rb);
				 }
			 }
		 }

		 return citas;
	 }

	 public Vector<ResultadoBusqueda> buscarCitaPorNH(int nh){

		 Vector<ResultadoBusqueda> citas = new Vector<ResultadoBusqueda>();

		 Cita c = this.sinAsignarDa.get(nh);
		 if( c!=null){
			 ResultadoBusqueda rb = new ResultadoBusqueda(c, null, this, false);
			 citas.add(rb);
		 }


		 c = this.sinAsignarDaEspecial.get(nh);
		 if( c!=null){
			 ResultadoBusqueda rb = new ResultadoBusqueda(c, null, this, true);
			 citas.add(rb);
		 }




		 for(Lista l:this.listas.values()){
			 c = l.getAsignados().get(nh);
			 if( c!=null){
				 ResultadoBusqueda rb = new ResultadoBusqueda(c, null, this, false);
				 citas.add(rb);
			 }

			 c = l.getEspeciales().get(nh);
			 if( c!=null){
				 ResultadoBusqueda rb = new ResultadoBusqueda(c, null, this, true);
				 citas.add(rb);
			 }
		 }

		 return citas;
	 }


	 public void crearLista(int dia, int max){
		 Lista l = new Lista(dia, this.mes, this.ao, max);
		 this.listas.put(dia, l);
	 }

	 public boolean borrarLista(int dia){
		 boolean borrado = false;

		 Lista l = this.listas.remove(dia);
		 if(l!= null){
			 borrado = true;
			 for(Cita c:l.getAsignados().values()){
				 this.sinAsignarDa.put(c.getNumeroHistoria(), c);
			 }

			 for(Cita c:l.getEspeciales().values()){
				 this.sinAsignarDaEspecial.put(c.getNumeroHistoria(), c);
			 }
		 }

		 return borrado;
	 }


	 public String cambiarListaDeDia(int diaIni, int diaFin) throws ListaNoCambiadaException{
		 if(!this.listas.containsKey(diaIni)){
			 throw new ListaNoCambiadaException("No existe ninguna lista asigndad al dia "+diaIni);
		 }

		 if(this.listas.containsKey(diaFin)){
			 throw new ListaNoCambiadaException("Ya hay una lista asigndad al dia "+diaFin);
		 }

		 Lista l = this.listas.remove(diaIni);
		 l.setDia(diaFin);
		 this.listas.put(diaFin, l);

		 String mensaje = "Cambio realizado sin problemas";
		 if(diaIni < 15 && diaFin >= 15){
			 Vector<Cita> citas = new Vector<Cita>();
			 for(Cita c:l.getAsignados().values()){
				 if(c.getPreferencia() == Cita.APRINCIPIOS){
					 citas.add(c);
				 }
			 }
			 for(Cita c:l.getEspeciales().values()){
				 if(c.getPreferencia() == Cita.APRINCIPIOS){
					 citas.add(c);
				 }
			 }

			 if(citas.size()>0){
				 mensaje = "En la lista cambiada hay "+citas.size()+" personas que prefieren ser citadas a principios de mes, pero la lista se ha cambiado a la segunda quincena del mes.\n";
				 mensaje += "Sera combeniente cambiar la cita de estas personas:\n";
				 for(Cita c:citas){
					 mensaje+=c.getNombre()+"\t"+c.getNumeroHistoria()+"\n";
				 }
			 }

		 }else if(diaIni >= 15 && diaFin < 15 ){
			 Vector<Cita> citas = new Vector<Cita>();
			 for(Cita c:l.getAsignados().values()){
				 if(c.getPreferencia() == Cita.AFINALES){
					 citas.add(c);
				 }
			 }
			 for(Cita c:l.getEspeciales().values()){
				 if(c.getPreferencia() == Cita.AFINALES){
					 citas.add(c);
				 }
			 }

			 if(citas.size()>0){
				 mensaje = "En la lista cambiada hay "+citas.size()+" personas que prefieren ser citadas a finales de mes, pero la lista se ha cambiado a la primera quincena del mes.\n";
				 mensaje += "Sera combeniente cambiar la cita de estas personas:\n";
				 for(Cita c:citas){
					 mensaje+=c.getNombre()+"\t"+c.getNumeroHistoria()+"\n";
				 }
			 }
		 }

		 return mensaje;
	 }

	 public void cambiarMaxLista(int dia, int maxN) throws ListaNoCambiadaException{
		 if(!this.listas.containsKey(dia)){
			 throw new ListaNoCambiadaException("No existe ninguna lista asigndad al dia "+dia);
		 }

		 Lista l = this.listas.get(dia);
		 l.setCantidadMaxima(maxN);
		 int nasignados = l.getNumeroAsignados();
		 Vector<Integer> keys = new Vector<Integer>();
		 keys.addAll(l.getAsignados().keySet());
		 for(int i = 0; i<keys.size() && nasignados>maxN ;i++){
			 int nh = keys.get(i);
			 Cita c = l.borrarAsignado(nh);
			 this.sinAsignarDa.put(c.getNumeroHistoria(), c);
			 nasignados = l.getNumeroAsignados();
		 }


	 }


	 public String organizarAsignados() throws DemasiadasCitasException{

		 int totalCitas = this.sinAsignarDa.size();

		 int totalHuecos = 0;
		 for(Lista l: listas.values()){
			 int aux = l.getCantidadMaxima()-l.getNumeroAsignados();
			 totalHuecos = totalHuecos+aux;
		 }

		 if(totalCitas>totalHuecos){
			 throw new DemasiadasCitasException("El nmero de personas citadas este mes excede el mximo de huecos en las listas creadas. Intente crear ms listas o aumentar el tamao mximo de alguna lista.\nTotal personas citadas: "+totalCitas+"\nTotal huecos disponibles: "+totalHuecos);
		 }

		 
		 for(Integer nh:this.getSinAsignarDa().keySet()){
			 for(Lista l:this.getListas().values()){
				 if(l.getAsignados().containsKey(nh)){
					 throw new DemasiadasCitasException("El paciente con nmero de historia "+nh+" est citado para "+this.mes+" de "+this.ao+" dos veces. 1 Sin un da concreto y  2 el da "+l.getDia()+" de forma normal. Por favor, antes de organizar las citas automticamente asigne manualmente la cita sin da a un da concreto o borrela.");
				 }
				 if(l.getEspeciales().containsKey(nh)){
					 throw new DemasiadasCitasException("El paciente con nmero de historia "+nh+" est citado para "+this.mes+" de "+this.ao+" dos veces. 1 Sin un da concreto y  2 el da "+l.getDia()+" de forma especial. Por favor, antes de organizar las citas automticamente asigne manualmente la cita sin da a un da concreto o borrela.");
				 }
			 }
		 }


		 Vector<Cita> citasDaigual = new Vector<Cita>();
		 Vector<Cita> citasAPrincipios = new Vector<Cita>();
		 Vector<Cita> citasAFinales = new Vector<Cita>();

		 Vector<Cita> citasV = new Vector<Cita>();
		 Vector<Object[]> citasV2 = new Vector<Object[]>();

		 for(Cita c:this.sinAsignarDa.values()){
			 citasV.add(c);
			 if(c.getPreferencia() == Cita.DAIGUAL){
				 citasDaigual.add(c);
			 }else if(c.getPreferencia() == Cita.APRINCIPIOS){
				 citasAPrincipios.add(c);
			 }else{
				 citasAFinales.add(c);
			 }
		 }
		 this.sinAsignarDa.clear();


		 //Hacemos una asignacin en anchura para determinar el nmero de citas de cada lista (esta asignacin es un simulacro y no ser la asignacin final)
		 Hashtable<Integer,Integer> diasInfo = new Hashtable<Integer, Integer>();
		 while(citasV.size()>0){

			 Lista aux = null;

			 for(Lista l:this.listas.values()){
				 if(l.hayHueco()){
					 if(aux == null){
						 aux = l;
					 }else if(aux.getNumeroAsignados() > l.getNumeroAsignados()){
						 aux = l;
					 }
				 }
			 }

			Cita c = citasV.remove(0);
			try {
				int dia = aux.getDia();
				if(!diasInfo.containsKey(dia)){
					diasInfo.put(dia, 1);
				}else{
					int nc = diasInfo.remove(dia);
					nc++;
					diasInfo.put(dia, nc);
				}
				 aux.addAsignado(c);
				 Object[] cv2 = new Object[2];
				 cv2[0] = c;
				 cv2[1] = aux;
				 citasV2.add(cv2);

			} catch (Exception e) {
				e.printStackTrace();
				citasV.add(c);
			}
		 }
		 for(Object []  c:citasV2){
			 this.borrarCita(((Cita)c[0]).getNumeroHistoria(),false,((Lista)c[1]).getDia());
		 }


		 //Ahora sabemos las citas que se han asignado cada da
		 //Primero asignamos los de aprincipios
		 TreeSet<Integer> dias = new TreeSet<Integer>();
		 dias.addAll(diasInfo.keySet());
		 for(Cita c:citasAPrincipios){
			 int dia = dias.first();
			 Lista l = this.listas.get(dia);
			 try {
				 l.addAsignado(c);
			 } catch (CitaConFechaNoAsignadaException e) {
				 // TODO Auto-generated catch block
				 e.printStackTrace();
			 } catch (CitaYaAsigndaException e) {
				 // TODO Auto-generated catch block
				 e.printStackTrace();
			 }
			 int nc = diasInfo.remove(dia);
			 nc--;
			 if(nc == 0){
				 dias.remove(dia);
			 }else{
				 diasInfo.put(dia, nc);
			 }
		 }

		//Ahora los de a finales
		 for(Cita c:citasAFinales){
			 int dia = dias.last();
			 Lista l = this.listas.get(dia);
			 try {
				l.addAsignado(c);
			} catch (CitaConFechaNoAsignadaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (CitaYaAsigndaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			 int nc = diasInfo.remove(dia);
			 nc--;
			 if(nc == 0){
				 dias.remove(dia);
			 }else{
				 diasInfo.put(dia, nc);
			 }
		 }

		//Ahora los de da igual
		 for(Cita c:citasDaigual){
			 int dia = dias.first();
			 Lista l = this.listas.get(dia);
			 try {
				l.addAsignado(c);
			} catch (CitaConFechaNoAsignadaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (CitaYaAsigndaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			 int nc = diasInfo.remove(dia);
			 nc--;
			 if(nc == 0){
				 dias.remove(dia);
			 }else{
				 diasInfo.put(dia, nc);
			 }
		 }


		 //Ahora hay que comprobar que los de a principios estn bien y los de a finales tambin
		 dias = new TreeSet<Integer>();
		 dias.addAll(this.listas.keySet());

		 //A principios
		 for(Lista l:listas.values()){
			 Vector<Integer> keys = new Vector<Integer>();
			 keys.addAll(l.getAsignados().keySet());
			 for(int ic = 0; ic<keys.size();ic++){
				 Cita c=l.getAsignados().get(keys.get(ic));


				 if(c.getPreferencia() == Cita.APRINCIPIOS && l.getDia()>15){
					 //Lo reasignamos si hay sitio
					 Lista aux = null;
					 for(int i = 0; i<dias.size() && aux == null;i++){
						 if(this.listas.get(dias.first()).hayHueco()){
							 aux = this.listas.get(dias.first());
						 }else{
							 dias.remove(dias.first());
							 i--;
						 }
					 }

					 if(aux != null){
						 l.borrarAsignado(c.getNumeroHistoria());
						 try {
							 aux.addAsignado(c);
						 } catch (CitaConFechaNoAsignadaException e) {
							 // TODO Auto-generated catch block
							 e.printStackTrace();
						 } catch (CitaYaAsigndaException e) {
							 // TODO Auto-generated catch block
							 e.printStackTrace();
						 }
					 }
				 }
			 }
		 }

		//A finales
		 for(Lista l:listas.values()){
			 Vector<Integer> keys = new Vector<Integer>();
			 keys.addAll(l.getAsignados().keySet());
			 for(int ic = 0; ic<keys.size();ic++){
				 Cita c=l.getAsignados().get(keys.get(ic));
				 
				 if(c.getPreferencia() == Cita.AFINALES && l.getDia()<=15){
					//Lo reasignamos si hay sitio
					Lista aux = null;
					 for(int i = 0; i<dias.size() && aux == null;i++){
						 if(this.listas.get(dias.last()).hayHueco()){
							 aux = this.listas.get(dias.last());
						 }else{
							 dias.remove(dias.last());
							 i--;
						 }
					 }

					 if(aux != null){
						 l.borrarAsignado(c.getNumeroHistoria());
						 try {
							aux.addAsignado(c);
						} catch (CitaConFechaNoAsignadaException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						} catch (CitaYaAsigndaException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					 }
				 }
			 }
		 }



		 //Crear mensaje
		 String mensaje = "";

		 Vector<Cita> aPricipio = new Vector<Cita>();
		 for(Lista l:listas.values()){
			 for(Cita c:l.getAsignados().values()){
				 if(c.getPreferencia() == Cita.APRINCIPIOS && l.getDia()>15){
					 aPricipio.add(c);
				 }
			 }
		 }

		 Vector<Cita> aFinales = new Vector<Cita>();
		 for(Lista l:listas.values()){
			 for(Cita c:l.getAsignados().values()){
				 if(c.getPreferencia() == Cita.AFINALES && l.getDia()<=15){
					 aFinales.add(c);
				 }
			 }
		 }

		 if(aPricipio.size()==0 && aFinales.size()==0){
			 mensaje = "Citas organizadas correctamente";
		 }else{
			 mensaje = "Las siguientes citas no han sido asignada teniendo en cuenta las preferencias de los pacientes.\n";
			 if(aPricipio.size()>0){
				 mensaje+="Pacientes asignados en la segunda quincena con preferencia de ser asigndos a principios de mes:\n";
				 for(Cita c:aPricipio){
					 mensaje+=c.getNombre()+" - "+c.getNumeroHistoria()+"\n";
				 }
			 }
			 if(aFinales.size()>0){
				 mensaje+="Pacientes asignados en la primera quincena con preferencia de ser asigndos a finales de mes:\n";
				 for(Cita c:aFinales){
					 mensaje+=c.getNombre()+" - "+c.getNumeroHistoria()+"\n";
				 }
			 }
		 }


		 return mensaje;

	 }


	 public void asignarDaACitaSinDa(int nh, int dia) throws CitaNoExisteException, CitaConFechaNoAsignadaException, CitaYaAsigndaException{
		 Cita c = null;
		 boolean especial;
		 c = this.sinAsignarDaEspecial.get(nh);
		 if(c == null){
			 c = this.sinAsignarDa.get(nh);
			 if(c == null){
				 throw new CitaNoExisteException("La persona con el nmero de historia "+nh+" no se encuentra citada sin fecha");
			 }else{
				 especial = false;
			 }
		 }else{
			 especial = true;
		 }

		 Lista l = this.listas.get(dia);

		 if(l == null){
			 throw new CitaConFechaNoAsignadaException("No hay ninguna lista asignda al "+dia+" de "+this.mes+" de "+this.ao);
		 }else{
			 if(especial){
				 l.addEspecial(c);
				 this.sinAsignarDaEspecial.remove(nh);
			 }else{
				 l.addAsignado(c);
				 this.sinAsignarDa.remove(nh);
			 }

		 }

	 }

	public String getMes() {
		return mes;
	}

	public void setMes(String mes) {
		this.mes = mes;
	}

	public int getAo() {
		return ao;
	}

	public void setAo(int ao) {
		this.ao = ao;
	}

	public Hashtable<Integer, Cita> getSinAsignarDa() {
		return sinAsignarDa;
	}

	public Hashtable<Integer, Cita> getSinAsignarDaEspecial() {
		return sinAsignarDaEspecial;
	}

	public Hashtable<Integer, Lista> getListas() {
		return listas;
	}

}
