package dominio.reportes;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.print.Book;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Vector;

import javax.print.attribute.PrintRequestAttributeSet;
import javax.swing.table.DefaultTableModel;

import com.jrefinery.report.Band;
import com.jrefinery.report.ElementAlignment;
import com.jrefinery.report.Group;
import com.jrefinery.report.GroupFooter;
import com.jrefinery.report.GroupHeader;
import com.jrefinery.report.ItemFactory;
import com.jrefinery.report.JFreeReport;
import com.jrefinery.report.ReportProcessingException;
import com.jrefinery.report.ShapeElement;
import com.jrefinery.report.TextElement;
import com.jrefinery.report.preview.PreviewDialog;
import com.jrefinery.report.targets.style.ElementStyleSheet;
import com.jrefinery.report.util.PageFormatFactory;

import dominio.Cita;
import dominio.Control;
import dominio.Lista;
import dominio.Mes;
import dominio.Printer;

public class ReportePrinter {
	private Control c;

	public ReportePrinter(Control c){
		this.c = c;
	}

	public boolean imprimirMes(String mes, int ao, int citasOption, boolean printComments, PageFormat pf, Lista l){
		boolean retorno = true;

		Mes m = c.getMes(mes, ao);
		if(m!=null){
			PrinterJob printerjob = PrinterJob.getPrinterJob();


			JFreeReport jfreereport = createReports(m,citasOption,printComments, pf,l);


			Book book = new Book();
			try {

				Printer printer = new Printer(jfreereport, pf);
				int i = printer.getNPages();

				for (int j = 0; j < i; j++){
					book.append(printer, pf);
				}



				printerjob.setPageable(book);

				if (printerjob.printDialog()){
					printerjob.print();
				}
			} catch (PrinterException printerexception) {
				printerexception.printStackTrace();
			}


		}else{
			retorno = false;
		}
		return retorno;
	}

	private JFreeReport createReports(Mes m, int opCitas, boolean com, PageFormat pf, Lista lista) {

		Vector<JFreeReport> retorno = new Vector<JFreeReport>();
		Vector<String[]> datosV = new Vector<String[]>();


		if(lista!=null){
			if(opCitas == Control.CITAS_NORMALES || opCitas == Control.CITAS_TODAS){
				for(Cita c:lista.getAsignados().values()){
					String [] s = new String [5];
					s[0] = lista.getDia()+"";
					s[1] = c.getNombre();
					s[2] = c.getNumeroHistoria()+"";
					s[3] = c.getComment();
					s[4] = "";
					datosV.add(s);
				}
			}

			if(opCitas == Control.CITAS_ESPECIALES || opCitas == Control.CITAS_TODAS){
				for(Cita c:lista.getEspeciales().values()){
					String [] s = new String [5];
					s[0] = lista.getDia()+"";
					s[1] = c.getNombre();
					s[2] = c.getNumeroHistoria()+"";
					s[3] = c.getComment();
					s[4] = "Cita especial";
					datosV.add(s);
				}
			}
		}else{
			//Primero un reporte para las citas sin da

			if(opCitas == Control.CITAS_NORMALES || opCitas == Control.CITAS_TODAS){
				for(Cita c:m.getSinAsignarDa().values()){
					String [] s = new String [5];
					s[0] = "Sin da";
					s[1] = c.getNombre();
					s[2] = c.getNumeroHistoria()+"";
					s[3] = c.getComment();
					s[4] = "";

					datosV.add(s);
				}
			}

			if(opCitas == Control.CITAS_ESPECIALES || opCitas == Control.CITAS_TODAS){
				for(Cita c:m.getSinAsignarDaEspecial().values()){
					String [] s = new String [5];
					s[0] = "Sin da";
					s[1] = c.getNombre();
					s[2] = c.getNumeroHistoria()+"";
					s[3] = c.getComment();
					s[4] = "Cita especial";
					datosV.add(s);
				}
			}

			//Y luego un reporte para cada una de las listas
			Vector<Integer> keys = new Vector<Integer>();
			keys.addAll(m.getListas().keySet());
			Collections.sort(keys);

			for(int k:keys){
				Lista l = m.getListas().get(k);
				if(l!=null){

					if(opCitas == Control.CITAS_NORMALES || opCitas == Control.CITAS_TODAS){
						for(Cita c:l.getAsignados().values()){
							String [] s = new String [5];
							s[0] = l.getDia()+"";
							s[1] = c.getNombre();
							s[2] = c.getNumeroHistoria()+"";
							s[3] = c.getComment();
							s[4] = "";
							datosV.add(s);
						}
					}

					if(opCitas == Control.CITAS_ESPECIALES || opCitas == Control.CITAS_TODAS){
						for(Cita c:l.getEspeciales().values()){
							String [] s = new String [5];
							s[0] = l.getDia()+"";
							s[1] = c.getNombre();
							s[2] = c.getNumeroHistoria()+"";
							s[3] = c.getComment();
							s[4] = "Cita especial";
							datosV.add(s);
						}
					}



				}
			}
		}




		String [] columnas = {"dia","nombre", "nh", "comentarios", "especial"};
		String [][] datos = new String[datosV.size()][];
		for(int i = 0; i<datosV.size();i++){
			datos[i] = datosV.get(i);
		}
		DefaultTableModel model	= new DefaultTableModel(datos, columnas);

		return this.createReport(model,m.getMes(), m.getAo(), com, pf);


	}


	public JFreeReport createReport(DefaultTableModel model, String mes, int ao, boolean comments, PageFormat pf){
		JFreeReport jfreereport = new JFreeReport();
		jfreereport.setName("Reporte de "+mes+" de "+ao+".");

		Font fph = new Font("", Font.BOLD, 14);
		Font fgh = new Font("", Font.PLAIN, 12);
		Font fgh2 = new Font("", Font.BOLD, 8);
		Font fdl = new Font("", Font.PLAIN, 8);
		Font fc = new Font("", Font.ITALIC, 8);
		Font fdia = new Font("", Font.BOLD, 12);

		double anchoPagina = pf.getWidth();

		String textNH = "Nmero de historia";

		double anchoNH = textNH.length() * 4.5;
		double anchoN;
		double anchoC;
		if(comments){
			anchoN = Math.max((anchoPagina-anchoNH)/2, 200);
			anchoC = anchoPagina-(anchoNH+anchoN);
		}else{
			anchoN = anchoPagina-anchoNH;
			anchoC = 0;
		}




		//Cabecera de las pginas


		TextElement lph = ItemFactory.createLabelElement("lph",
				new Rectangle2D.Double(0.0,0.0,500.0,15.0),
				Color.black,
				ElementAlignment.LEFT.getOldAlignment(),
				fph,
				mes +" de "+ao);
		jfreereport.getPageHeader().addElement(lph);
		ShapeElement linea = ItemFactory.createLineShapeElement("linea",
				Color.black,
				new BasicStroke(1),
				new Line2D.Double(0.0, 16.0, anchoPagina, 16.0));

		jfreereport.getPageHeader().addElement(linea);


		//Creamos el grupo
		Band gh = ItemFactory.createGroupHeader(20, null, Color.black, true);
		Band gf = ItemFactory.createGroupFooter(20, null, Color.black);

		ArrayList<String> colums = new ArrayList<String>();
		colums.add("dia");
		Group group = ItemFactory.createGroup("grupo", colums, (GroupFooter)gf, (GroupHeader)gh);

		jfreereport.addGroup(group);

		//Aadimos los elementos

		ShapeElement lineData = ItemFactory.createLineShapeElement("lineData", Color.black, new BasicStroke((float)0.2), new Line2D.Double(0, 0, anchoPagina, 0));
		jfreereport.getItemBand().addElement(lineData);

		//Nombre

		//Groupheader
		TextElement labelNombre = ItemFactory.createLabelElement("lN",
				new Rectangle2D.Double(85.0, 20.0, anchoN, 20.0),
				Color.black,
				ElementAlignment.LEFT.getOldAlignment(),
				fgh2,
				"Nombre");
		gh.addElement(labelNombre);


		//data
		TextElement textelement = ItemFactory.createStringElement("nombres",
				new Rectangle2D.Double(85.0, 5.0, anchoN, 20.0),
				Color.black,
				ElementAlignment.LEFT.getOldAlignment(),
				fdl,
				"",
				"nombre");
		textelement.getStyle().setBooleanStyleProperty(ElementStyleSheet.DYNAMIC_HEIGHT, true);
		jfreereport.getItemBand().addElement( textelement);





		//Nmeor de historia

		//GroupHeader
		TextElement labelNH = ItemFactory.createLabelElement("lNh",
				new Rectangle2D.Double(0.0, 20.0, anchoNH, 20.0),
				Color.black,
				ElementAlignment.LEFT.getOldAlignment(),
				fgh2,
				"Nmero de Historia");
		gh.addElement(labelNH);

		//data
		TextElement textelement2 = ItemFactory.createStringElement("T2",
				new Rectangle2D.Double(5.0, 5.0, anchoNH, 20.0),
				Color.black,
				ElementAlignment.LEFT.getOldAlignment(),
				fdl,
				"",
				"nh");
		jfreereport.getItemBand().addElement( textelement2);



		if(comments){
			//Comentario

			//groupHeader
			TextElement labelC = ItemFactory.createLabelElement("lC",
					new Rectangle2D.Double(290.0, 20.0, anchoC, 20.0),
					Color.black,
					ElementAlignment.LEFT.getOldAlignment(),
					fgh2,
					"Comentarios");
			gh.addElement(labelC);

			//data
			TextElement textelement3 = ItemFactory.createStringElement("T3",
					new Rectangle2D.Double(290.0, 5.0, anchoC, 20.0),
					Color.black,
					ElementAlignment.LEFT.getOldAlignment(),
					fc,
					"",
					"comentarios");
			textelement3.getStyle().setBooleanStyleProperty(ElementStyleSheet.DYNAMIC_HEIGHT, true);
			jfreereport.getItemBand().addElement(textelement3);

		}



		//GroupHeader - da
		String text1 = "Da  ";
		gh.addElement(ItemFactory.createLabelElement("L1",
				new Rectangle2D.Double(0.0, 5.0,text1.length()*6,25.0),
				Color.black,
				ElementAlignment.RIGHT.getOldAlignment(),
				fdia,
				text1));
		gh.addElement(ItemFactory.createStringElement("dia",
				new Rectangle2D.Double(text1.length()*6, 5.0, 100, 25.0),
				Color.black,
				ElementAlignment.LEFT.getOldAlignment(),
				fdia,
				"",
				"dia"));




		jfreereport.setData(model);




		return jfreereport;
	}


}
