package pruebas.cobertura.dominio;


import java.io.File;
import java.lang.reflect.Method;
import java.util.Calendar;
import java.util.Vector;

import junit.framework.TestCase;

import mockit.Mockit;


import persistencia.Persistencia;
import pruebas.cobertura.MyCalendar;
import pruebas.cobertura.PersistenciaStubsInterface;
import pruebas.cobertura.PersistenciaStubsInterface2;
import pruebas.cobertura.VentanaStub;
import dominio.Control;
import dominio.Mes;
import dominio.ResultadoBusqueda;

public class InitialTests extends TestCase{


	Control c;
	VentanaStub v;
	
	public void setUp(){
		
		
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface.class);
		Mockit.setUpMock(Calendar.class,MyCalendar.class);
		c = null;
		v = null;
		
		v = new VentanaStub();
		c = new Control(v);
		
		
	}
	
	public void tearDown(){
		Mockit.tearDownMocks(Persistencia.class);
		Mockit.tearDownMocks(Calendar.class);
	}
	
	
	public void testCrearLista() {
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = 15;
		l[1] = 15;
		lista.add(l);
		c.crearListas(2012, "Junio", lista);
		assertTrue(c.getMes("Junio", 2012).getListas().containsKey(15));
	}
	
	public void testCrearListaErrorDB() {
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = 15;
		l[1] = 15;
		lista.add(l);
		c.crearListas(2012, "Junio", lista);
		assertTrue(c.getMes("Junio", 2012).getListas().containsKey(15));
	}
	
	public void testCrearListaExistente() {
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = 15;
		l[1] = 15;
		lista.add(l);
		c.crearListas(2011, "Enero", lista);
		assertTrue(c.getMes("Enero", 2011).getListas().size()==2);
	}
	
	public void testCrearListaNocogen() {
		
		for(int i = 14;i<16;i++){
			c.borrarLista(2011, "Enero", i);
		}
		
		
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = 5;
		l[1] = 5;
		lista.add(l);
		boolean r = c.crearListas(2011, "Enero", lista);
		assertTrue(c.getMes("Enero", 2011).getListas().size()==0);
		assertTrue(r == false);
		
	}
	
	
	public void testAddCitaConFecha(){
		
		c.addCitaConFecha(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 15);
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getAsignados().containsKey(1234));
	}
	
	public void testAddCitaConFechaMesVacio(){	
		c.addCitaConFecha(1234, "pedro lopez", "no hay comentario", 0, 2012, "Enero", 15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaConFechaYaAsignada(){
		
		c.addCitaConFecha(115, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaConFechaErrorGuardarDataBase(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.addCitaConFecha(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 15);
		assertTrue(v.errors.size()==0);
	}
	
	public void testAddCItaSinFecha() {
		c.addCitaSinFecha(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero");
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().containsKey(1234));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().get(1234).getNombre().equals("pedro lopez"));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().get(1234).getComment().equals("no hay comentario"));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().get(1234).getPreferencia()==0);
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().get(1234).getNumeroHistoria() == 1234);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
		
	}
	
	public void testAddCItaSinFechaEnMesVacio() {
		c.addCitaSinFecha(1234, "pedro lopez", "no hay comentario", 0, 2012, "Enero");
		assertTrue(c.getMes("Enero", 2012).getSinAsignarDa().containsKey(1234));
		
	}
	
	public void testAddCItaSinFechaYaAsignada() {
		c.addCitaSinFecha(514, "test", "no hay comentario", 0, 2011, "Enero");
		assertTrue(v.errors.size()==1);
		
	}
	
	public void testAddCItaErrorGuardarDatabase() {
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.addCitaSinFecha(5123, "test", "no hay comentario", 0, 2011, "Enero");
		assertTrue(v.errors.size()==0);
		
	}

	public void testAddCItaEspecial() {
		
		c.addCitaEspecial(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 15);
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getEspeciales().containsKey(1234));
		
	}
	
	public void testAddCitaEspecialConFechaMesVacio(){	
		
		c.addCitaEspecial(1234, "pedro lopez", "no hay comentario", 0, 2012, "Enero", 15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaEspecialConFechaYaAsignada(){
		
		c.addCitaEspecial(215, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaEspecialConFechaErrorGuardarDataBase(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.addCitaEspecial(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 15);
		assertTrue(v.errors.size()==0);
	}
	
	
	
	public void testAsignarCItaADa() {
		
		c.asignarDiaCita(515, 2011, "Enero", 15);
		
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getAsignados().containsKey(515));
		assertTrue(!c.getMes("Enero", 2011).getSinAsignarDa().containsKey(515));
	}
	public void testAsignarCItaADaMesVacio(){	
		c.asignarDiaCita(515, 2012, "Enero",15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaYaAsignada(){
		
		
		c.addCitaSinFecha(115, "", "", 0, 2011, "Enero");
		c.asignarDiaCita(115, 2011, "Enero", 15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaSinLista(){

		c.asignarDiaCita(515, 2011, "Enero", 2);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaErrorGuardarDataBase(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.asignarDiaCita(515, 2011, "Enero", 15);
		assertTrue(v.errors.size()==0);
	}
	
	
	
	
	public void testBorrarCitaNormalSinFecha(){
		
		c.borrarCita("Enero", 2011, 515, false, -1);
		assertTrue(!c.getMes("Enero", 2011).getSinAsignarDa().containsKey(515));
	}
	
	public void testBorrarCitaNormalSinFechaMesVacio(){
		
		boolean r = c.borrarCita("Enero", 2012, 515, false, -1);
		assertTrue(r == false);
	}
	
	public void testBorrarCitaNormalSinFechaErrorGuardarDataBase(){
		
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.borrarCita("Enero", 2011, 515, false, -1);
		assertTrue(!c.getMes("Enero", 2011).getSinAsignarDa().containsKey(515));
		assertTrue(v.errors.size()==0);
	}
	
	public void testBorrarCitaNormalConFecha(){
		
		c.borrarCita("Enero", 2011, 115, false, 15);
		assertTrue(!c.getMes("Enero", 2011).getListas().get(15).getAsignados().containsKey(115));
	}
	
	public void testBorrarCitaEspecialConFecha(){
		
		c.borrarCita("Enero", 2011, 215, true, 15);
		assertTrue(!c.getMes("Enero", 2011).getListas().get(15).getEspeciales().containsKey(215));
	}

	public void testBorrarLista(){
		
		c.borrarLista(2011, "Enero", 15);
		
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(15));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==7);
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDaEspecial().size()==1);
	}
	
	public void testBorrarListaNoExistente(){
		
		boolean r = c.borrarLista(2011, "Enero", 5);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(15));
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(14));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==6);
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDaEspecial().size()==0);
		assertTrue(v.errors.size()==1);
		assertTrue(r == false);
	}
	
	public void testBorrarListaMesVacio(){
		
		boolean r = c.borrarLista(2012, "Enero", 15);
		
		assertTrue(v.errors.size()==1);
		assertTrue(r == false);
		
	}
	
	public void testBorrarListaErrorDB(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.borrarLista(2011, "Enero", 15);
		
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(15));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==7);
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDaEspecial().size()==1);
	}
	
	
	public void testBuscarPorNH(){
		
		Vector<ResultadoBusqueda> res = c.buscarCitasPorNH(315);
		assertTrue(res.size()==1);
		assertTrue(res.get(0).getC().getNombre().equals("test2"));
	}
	
	public void testBuscarPorNombre(){

		Vector<ResultadoBusqueda> res = c.buscarCitasPorNombre("test");
		assertTrue(res.size()==10);
		assertTrue(res.get(0).getC().getNombre().contains("test"));
		assertTrue(res.get(9).getC().getNombre().contains("test"));
	}
	
	public void testcambiarDataBase(){
		
		try {
			
			c.cambiarDataBase(new File("./NuevaDataBase"));
			
			assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==6);
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			fail();
			e.printStackTrace();
		}
		
		
	}
	
	public void testCambiarListaDeDia(){


		c.cambiarListaDeDa(2011, "Enero", 15, 1);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(1));
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(15));
	}
	
	public void testCambiarListaDeDiaMesVacio(){


		c.cambiarListaDeDa(2012, "Enero", 15, 1);
		assertTrue(v.errors.size()==1);
		
	}
	
	public void testCambiarListaDeDiaErrorDB(){

		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.cambiarListaDeDa(2011, "Enero", 15, 1);
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(1));
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(15));
		
	}
	
	public void testCambiarListaDeDiaError(){

		boolean r = c.cambiarListaDeDa(2011, "Enero", 15, 14);
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
		
	}
	
	public void testCambiarMaxLista(){

		c.cambiarMaxLista(2011, "Enero", 15, 3);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(15));
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getCantidadMaxima()==3);
	}
	
	public void testCambiarMaxListaMesVacio(){

		c.cambiarMaxLista(2012, "Enero", 15, 3);
		assertTrue(v.errors.size()==1);
	}
	
	public void testCambiarMaxListaErrorDB(){

		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.cambiarMaxLista(2011, "Enero", 15, 3);

		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(15));
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getCantidadMaxima()==3);

	}
	
	public void testCambiarMaxListaError(){

		boolean r = c.cambiarMaxLista(2011, "Enero", 1, 3);
		
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
	}
	
	public void testGetMes(){
		
		
		
		Mes m = c.getMes("Enero", 2011);
		assertTrue(m.getListas().containsKey(15));
		assertTrue(m.getListas().get(15).getCantidadMaxima()==10);
		
		Mes m2 = c.getMes("Febrero", 2011);
		assertTrue(!m2.getListas().containsKey(15));
		
		
	}

	
	public void testGetMesesToStore(){
		
		int me = c.getMesesToStore();
		assertTrue(me == 24);
		
	}
	public void testGetPathCitas(){
		
		String me = c.getPathDBCitas();
		assertTrue(me.equals("."));
		
	}
	
	public void testsetConf(){
		
		
		c.setConfiguration(17);
		assertTrue(c.getMesesToStore()==17);
	}
	
	public void testOrganizarAsignados(){
		
		
		c.oganizarAsignados(2011, "Enero");
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==0);
		
		
		
	}
	
	public void testOrganizarAsignadosMesVacio(){

		boolean r = c.oganizarAsignados(2012, "Enero");
		assertTrue(r == true);
	}
	
	public void testOrganizarAsignadosNoCogen(){

		for(int i = 14;i<16;i++){
			c.getMes("Enero", 2011).borrarLista(i);
		}
		boolean r = c.oganizarAsignados(2011, "Enero");
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
	}
	
	
	public void testOrganizarAsignadosErrorDB(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.oganizarAsignados(2011, "Enero");
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==0);
		assertTrue(v.errors.size()==0);
	}
	
	
	public void testBorrarExtraData(){
		
		
		c.getMes("Enero", 2010);
		c.getMes("Febrero", 2010);
		c.getMes("Marzo", 2010);
		c.getMes("Abril", 2010);
		c.getMes("Mayo", 2010);
		c.getMes("Junio", 2010);
		c.getMes("Julio", 2010);
		c.getMes("Agosto", 2010);
		c.getMes("Septiembre", 2010);
		c.getMes("Octubre", 2010);
		c.getMes("Noviembre", 2010);
		c.getMes("Diciembre", 2010);
		
		c.setConfiguration(12);
		
		try {
			Method m = c.getClass().getDeclaredMethod("borrarExtraData");
			m.setAccessible(true);
			
				c.setConfiguration(11);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Enero"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Febrero"));
				
				c.setConfiguration(10);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Febrero"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Marzo"));
				
				c.setConfiguration(9);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Marzo"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Abril"));
				
				c.setConfiguration(8);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Abril"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Mayo"));
				
				c.setConfiguration(7);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Mayo"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Junio"));
				
				c.setConfiguration(6);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Junio"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Julio"));
				
				c.setConfiguration(5);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Julio"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Agosto"));
				
				c.setConfiguration(4);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Agosto"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Septiembre"));
				
				c.setConfiguration(3);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Septiembre"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Octubre"));
				
				c.setConfiguration(2);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Octubre"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Noviembre"));
				
				c.setConfiguration(1);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Noviembre"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2010Diciembre"));
				
				c.setConfiguration(0);
				m.invoke(c);
				assertTrue(!PersistenciaStubsInterface.meses.containsKey("2010Diciembre"));
				assertTrue(PersistenciaStubsInterface.meses.containsKey("2011Enero"));
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
	}
	
	
}
