package pruebas.cobertura.dominio;



import junit.framework.TestCase;


import dominio.Cita;
import dominio.Lista;
import dominio.exceptions.CitaConFechaNoAsignadaException;
import dominio.exceptions.CitaYaAsigndaException;

public class TestCubrirLista extends TestCase{

	public void testListaSEtGets(){
		
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		assertTrue(l.getAsignados().size()==0);
		assertTrue(l.getMes().equals("Enero"));
		assertTrue(l.getAo()==2011);
		assertTrue(l.getCantidadMaxima()==10);
		assertTrue(l.getDia()==5);
		assertTrue(l.getEspeciales().size()==0);
		assertTrue(l.getNumeroAsignados() == 0);
		assertTrue(l.getNumeroEspeciales() == 0);
		assertTrue(l.getNumeroTotal() == 0);

		l.setAo(2012);
		assertTrue(l.getAo()==2012);
		
		l.setCantidadMaxima(15);
		assertTrue(l.getCantidadMaxima()==15);
		
		l.setDia(2);
		assertTrue(l.getDia()==2);
		
		l.setMes("Febrero");
		assertTrue(l.getMes().equals("Febrero"));
		
		
	}

	
	public void testAddAsignado() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException{
		
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		Cita c = new Cita("n", 1, "c", 0);
		
		l.addAsignado(c);
		assertTrue(l.getAsignados().size()==1);
		
		l.borrarAsignado(1);
		assertTrue(l.getAsignados().size()==0);
	
	}
	
	public void testAddEspecial() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException{
		
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		Cita c = new Cita("n", 1, "c", 0);
		
		l.addEspecial(c);
		assertTrue(l.getEspeciales().size()==1);
		
		l.borrarEspecial(1);
		assertTrue(l.getEspeciales().size()==0);
	
	}
	
	public void testAddAsignadoEx1(){
		
		Lista l = new Lista(5, "Enero", 2011, 0);
		
		Cita c = new Cita("n", 1, "c", 0);
		
		try {
			l.addAsignado(c);
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			assertTrue(l.getAsignados().size()==0);
		} catch (CitaYaAsigndaException e) {
			fail();
		}
		
	}
	
	public void testAddAsignadoEx2(){
		
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		Cita c = new Cita("n", 1, "c", 0);
		
		try {
			l.addAsignado(c);
			l.addAsignado(c);
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			fail();
			
		} catch (CitaYaAsigndaException e) {
			assertTrue(l.getAsignados().size()==1);
		}
		
	}
	
	public void testAddAsignadoEx3(){
		
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		Cita c = new Cita("n", 1, "c", 0);
		
		try {
			l.addEspecial(c);
			l.addAsignado(c);
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			fail();
			
		} catch (CitaYaAsigndaException e) {
			assertTrue(l.getAsignados().size()==0);
			assertTrue(l.getEspeciales().size()==1);
		}
		
	}
	
	
	public void testAddEspecialEx1(){
		
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		Cita c = new Cita("n", 1, "c", 0);
		
		try {
			l.addEspecial(c);
			l.addEspecial(c);
			fail();
		} catch (CitaYaAsigndaException e) {
			assertTrue(l.getEspeciales().size()==1);
		}
		
	}
	
	public void testAddEspecialEx2(){
		
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		Cita c = new Cita("n", 1, "c", 0);
		
		try {
			
			l.addAsignado(c);
			l.addEspecial(c);
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			fail();
			
		} catch (CitaYaAsigndaException e) {
			assertTrue(l.getAsignados().size()==1);
			assertTrue(l.getEspeciales().size()==0);
		}
		
	}
	
	public void testHayHueco(){
		Lista l = new Lista(5, "Enero", 2011, 10);
		
		boolean r = l.hayHueco();
		assertTrue(r == true);
	}
	
	public void testNoHayHueco(){
		Lista l = new Lista(5, "Enero", 2011, 0);
		
		boolean r = l.hayHueco();
		assertTrue(r == false);
	}
	
	
	
	
}
