package pruebas.cobertura.dominio;

import java.util.Vector;

import dominio.Cita;
import dominio.Mes;
import dominio.ResultadoBusqueda;
import dominio.exceptions.CitaConFechaNoAsignadaException;
import dominio.exceptions.CitaNoExisteException;
import dominio.exceptions.CitaYaAsigndaException;
import dominio.exceptions.DemasiadasCitasException;
import dominio.exceptions.ListaNoCambiadaException;
import junit.framework.TestCase;

public class TestCubrirMes extends TestCase{

	
	Mes m;
	
	protected void setUp(){
		m=null;
		
		m = new Mes("Enero", 2011);
		
		for(int i = 5;i<25;i++){
			m.crearLista(i, 10);
			try {
				m.asignarCitaConFecha(new Cita("test", 100+i, "testcoment", 0), i, false);
				m.asignarCitaConFecha(new Cita("tests", 200+i, "testcomens", 0), i, true);
				
				
				m.asignarCitaSinFecha(new Cita("test", 500+i, "testcoment", 0));
				m.asignarCitaSinFecha(new Cita("test2", 300+i, "testcoment2", 1));
				m.asignarCitaSinFecha(new Cita("test2", 400+i, "testcoment3", 2));
				
				m.getSinAsignarDaEspecial().put(600+i,new Cita("testS", 600+i, "testcomentS", 0));
			} catch (CitaConFechaNoAsignadaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (CitaYaAsigndaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}
	
	
	public void testSetGets(){
		
		assertTrue(m.getAo()==2011);
		assertTrue(m.getMes().equals("Enero"));
		assertTrue(m.getListas().size()==20);
		assertTrue(m.getSinAsignarDa().size()==60);
		assertTrue(m.getSinAsignarDaEspecial().size() == 20);
		
		m.setAo(2012);
		assertTrue(m.getAo()==2012);
		
		m.setMes("Febrero");
		assertTrue(m.getMes().equals("Febrero"));
	}
	
	public void testAsignarCitaExistente(){
		
		try {
			m.asignarCitaSinFecha(new Cita("", 505, "", 0));
			fail();
		} catch (CitaYaAsigndaException e) {
			assertTrue(m.getSinAsignarDa().size()==60);
		}
		
	}
	
	public void testAsignarCitaConFechaListaVacia(){
		Mes m2 = new Mes("Enero", 2011);
		try {
			m2.asignarCitaConFecha(new Cita("", 1, "", 0), 1, false);
			fail();
		} catch (CitaYaAsigndaException e) {
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			assertTrue(m2.getSinAsignarDa().size()==0);
		}
		
	}
	
	public void testAsignarCitaConFechaListaMala(){
		
		try {
			m.asignarCitaConFecha(new Cita("", 1, "", 0), 1, false);
			fail();
		} catch (CitaYaAsigndaException e) {
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			assertTrue(m.getSinAsignarDa().size()==60);
		}
		
	}
	
	public void testBorrarCita(){
		
		m.borrarCita(505, false, -1);
		assertTrue(m.getSinAsignarDa().size()==59);
	}
	
	public void testBorrarCita2(){
		
		m.borrarCita(605, true, -1);
		assertTrue(m.getSinAsignarDaEspecial().size()==19);
	}
	
	public void testBorrarCita3(){
		
		m.borrarCita(105, false, 5);
		assertTrue(m.getListas().get(5).getAsignados().size()==0);
	}
	
	public void testBorrarCita4(){
		
		m.borrarCita(205, true, 5);
		assertTrue(m.getListas().get(5).getEspeciales().size()==0);
	}
	
	public void testBuscarCitaPorNombre(){
		Vector<ResultadoBusqueda> res = m.buscarCitaPorNombre("test");
		assertTrue(res.size() == 120);
	}
	
	public void testBuscarCitaPorNH(){
		Vector<ResultadoBusqueda> res = m.buscarCitaPorNH(605);
		assertTrue(res.size() == 1);
	}
	
	public void testBuscarCitaPorNH2(){
		Vector<ResultadoBusqueda> res = m.buscarCitaPorNH(105);
		assertTrue(res.size() == 1);
	}
	
	public void testBuscarCitaPorNH3(){
		Vector<ResultadoBusqueda> res = m.buscarCitaPorNH(205);
		assertTrue(res.size() == 1);
	}
	
	public void testBuscarCitaPorNH4(){
		Vector<ResultadoBusqueda> res = m.buscarCitaPorNH(505);
		assertTrue(res.size() == 1);
	}
	
	public void testBorrarLista(){
		boolean r = m.borrarLista(5);
		assertTrue(r == true);
		assertTrue(!m.getListas().containsKey(5));
	}
	
	public void testCambiarListaDeDia(){
		try {
			m.cambiarListaDeDia(5, 4);
			assertTrue(m.getListas().containsKey(4));
			assertTrue(!m.getListas().containsKey(5));
		} catch (ListaNoCambiadaException e) {
			fail();
		}
	}
	
	public void testCambiarListaDeDiaError1(){
		try {
			m.cambiarListaDeDia(4, 3);
			fail();
		} catch (ListaNoCambiadaException e) {
			assertTrue(!m.getListas().containsKey(4));
			assertTrue(!m.getListas().containsKey(3));
		}
	}
	
	public void testCambiarListaDeDiaError2(){
		try {
			m.cambiarListaDeDia(5, 6);
			fail();
		} catch (ListaNoCambiadaException e) {
			assertTrue(m.getListas().containsKey(5));
			assertTrue(m.getListas().containsKey(6));
		}
	}
	
	public void testCambiarListaDeDia2() throws DemasiadasCitasException, CitaYaAsigndaException{
		m.getListas().get(5).addEspecial(new Cita("", 123, "", Cita.APRINCIPIOS));
		m.organizarAsignados();
		try {
			String msg = m.cambiarListaDeDia(5, 26);
			assertTrue(m.getListas().containsKey(26));
			assertTrue(!m.getListas().containsKey(5));
			
			assertTrue(!msg.equals("Cambio realizado sin problemas"));
		} catch (ListaNoCambiadaException e) {
			fail();
		}
	}
	
	public void testCambiarListaDeDia3() throws DemasiadasCitasException, CitaYaAsigndaException{
		m.getListas().get(20).addEspecial(new Cita("", 123, "", Cita.AFINALES));
		m.organizarAsignados();
		try {
			String msg = m.cambiarListaDeDia(20, 4);
			assertTrue(m.getListas().containsKey(4));
			assertTrue(!m.getListas().containsKey(20));
			
			assertTrue(!msg.equals("Cambio realizado sin problemas"));
		} catch (ListaNoCambiadaException e) {
			fail();
		}
	}
	
	public void testCambiarMaxLista() throws ListaNoCambiadaException{
		m.cambiarMaxLista(5, 20);
		assertTrue(m.getListas().get(5).getCantidadMaxima()==20);
	}
	
	public void testCambiarMaxListaMala(){
		try {
			m.cambiarMaxLista(4, 20);
			fail();
		} catch (ListaNoCambiadaException e) {
			assertTrue(!m.getListas().containsKey(4));
		}
		
	}
	
	public void testCambiarMaxListaPoco() throws ListaNoCambiadaException, CitaConFechaNoAsignadaException, CitaYaAsigndaException{
		m.asignarCitaConFecha(new Cita("", 123, "", 0), 5, false);
		m.cambiarMaxLista(5, 1);
		assertTrue(m.getListas().get(5).getCantidadMaxima()==1);
		assertTrue(m.getSinAsignarDa().size()==61);
		assertTrue(m.getListas().get(5).getAsignados().size()==1);
	}
	
	public void testAsignarDiaAcita() throws CitaNoExisteException, CitaConFechaNoAsignadaException, CitaYaAsigndaException{
		m.asignarDaACitaSinDa(505, 5);
		assertTrue(m.getSinAsignarDa().size()==59);
		assertTrue(m.getListas().get(5).getAsignados().size()==2);
		
	}
	
	public void testAsignarDiaAcitaError1() {
		try {
			m.asignarDaACitaSinDa(12335, 5);
			fail();
		} catch (CitaNoExisteException e) {
			assertTrue(m.getSinAsignarDa().size()==60);
			assertTrue(m.getListas().get(5).getAsignados().size()==1);
		} catch (CitaConFechaNoAsignadaException e) {
			fail();
		} catch (CitaYaAsigndaException e) {
			fail();
		}
		
		
	}
	
	public void testAsignarDiaAcitaError2() {
		try {
			m.asignarDaACitaSinDa(505, 4);
			fail();
		} catch (CitaNoExisteException e) {
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			assertTrue(m.getSinAsignarDa().size()==60);
			assertTrue(m.getListas().get(5).getAsignados().size()==1);
		} catch (CitaYaAsigndaException e) {
			fail();
		}
		
		
	}
	
	public void testAsignarDiaAcitaEspecial() throws CitaNoExisteException, CitaConFechaNoAsignadaException, CitaYaAsigndaException{
		m.asignarDaACitaSinDa(605, 5);
		assertTrue(m.getSinAsignarDa().size()==60);
		assertTrue(m.getSinAsignarDaEspecial().size()==19);
		assertTrue(m.getListas().get(5).getEspeciales().size()==2);
		
	}
	
	
	public void testOrganizarCItasNOHueco(){
		for(int i = 5;i<25;i++){
			try {
				m.cambiarMaxLista(i, 3);
			} catch (ListaNoCambiadaException e) {
				fail();
			}
		}
		
		try {
			m.organizarAsignados();
			fail();
		} catch (DemasiadasCitasException e) {
			assertTrue(m.getSinAsignarDa().size()==60);
			assertTrue(m.getSinAsignarDaEspecial().size()==20);
		}
	}
	
	public void testOrganizarCItasError1(){
		
		Mes m2 = new Mes("Enero", 2011);
		
		m2.crearLista(5, 10);
		Cita c1 = new Cita("", 100, "", 0);
		Cita c2 = new Cita("", 100, "", 0);
		try {
			m2.asignarCitaConFecha(c1, 5, false);
		} catch (CitaConFechaNoAsignadaException e1) {
			fail();
		} catch (CitaYaAsigndaException e1) {
			fail();
		}
		
		
		try {
			m2.asignarCitaSinFecha(c2);
		} catch (CitaYaAsigndaException e1) {
			fail();
		}
		
		
		try {
			m2.organizarAsignados();
			fail();
		} catch (DemasiadasCitasException e) {
			assertTrue(e.getMessage().equals("El paciente con nmero de historia 100 est citado para Enero de 2011 dos veces. 1 Sin un da concreto y  2 el da 5 de forma normal. Por favor, antes de organizar las citas automticamente asigne manualmente la cita sin da a un da concreto o borrela."));
		}
	}
	
	public void testOrganizarCItasError2(){
		
		Mes m2 = new Mes("Enero", 2011);
		
		m2.crearLista(5, 10);
		Cita c1 = new Cita("", 100, "", 0);
		Cita c2 = new Cita("", 100, "", 0);
		try {
			m2.asignarCitaConFecha(c1, 5, true);
		} catch (CitaConFechaNoAsignadaException e1) {
			fail();
		} catch (CitaYaAsigndaException e1) {
			fail();
		}
		
		
		try {
			m2.asignarCitaSinFecha(c2);
		} catch (CitaYaAsigndaException e1) {
			fail();
		}
		
		
		try {
			m2.organizarAsignados();
			fail();
		} catch (DemasiadasCitasException e) {
			assertTrue(e.getMessage().equals("El paciente con nmero de historia 100 est citado para Enero de 2011 dos veces. 1 Sin un da concreto y  2 el da 5 de forma especial. Por favor, antes de organizar las citas automticamente asigne manualmente la cita sin da a un da concreto o borrela."));
		}
	}
	
	
	public void testOrganizarAprincipiosPRoblemas(){
		
		Mes m2 = new Mes("Enero", 2011);
		
		for(int i = 13;i<19;i++){
			m2.crearLista(i, 5);
			try {
				
				m2.asignarCitaSinFecha(new Cita("test", 500+i, "testcoment", Cita.APRINCIPIOS));
				m2.asignarCitaSinFecha(new Cita("test2", 300+i, "testcoment2", Cita.APRINCIPIOS));
				m2.asignarCitaSinFecha(new Cita("test2", 400+i, "testcoment3", Cita.APRINCIPIOS));
				
			}catch (CitaYaAsigndaException e) {
				fail();
			}
		}
		
		String men="";
		try {
			men = m2.organizarAsignados();
		} catch (DemasiadasCitasException e) {
			fail();
		}
		
		assertTrue(men.split("\n").length==5);
		
	}
	
	public void testOrganizarAFinalesPRoblemas(){
		
		Mes m2 = new Mes("Enero", 2011);
		
		for(int i = 13;i<19;i++){
			m2.crearLista(i, 5);
			try {
				
				m2.asignarCitaSinFecha(new Cita("test", 500+i, "testcoment", Cita.AFINALES));
				m2.asignarCitaSinFecha(new Cita("test2", 300+i, "testcoment2", Cita.AFINALES));
				m2.asignarCitaSinFecha(new Cita("test2", 400+i, "testcoment3", Cita.AFINALES));
				
			}catch (CitaYaAsigndaException e) {
				fail();
			}
		}
		
		String men="";
		try {
			men = m2.organizarAsignados();
		} catch (DemasiadasCitasException e) {
			fail();
		}
		
		assertTrue(men.split("\n").length==5);
		
	}
	
	
	
	
}
