package pruebas.mutacion.dominio;


import java.util.Calendar;
import java.util.Vector;

import junit.framework.TestCase;

import mockit.Mockit;


import persistencia.Persistencia;
import pruebas.cobertura.MyCalendar;
import pruebas.cobertura.PersistenciaStubsInterface2;
import pruebas.cobertura.PersistenciaStubsInterfaceNeg;
import pruebas.cobertura.VentanaStub;
import dominio.Control;
import dominio.Mes;
import dominio.ResultadoBusqueda;

public class InitialTestsNeg extends TestCase{


	
	Control c;
	VentanaStub v;
	
	public void setUp(){
		
		
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterfaceNeg.class);
		Mockit.setUpMock(Calendar.class,MyCalendar.class);
		c = null;
		v = null;
		
		v = new VentanaStub();
		c = new Control(v);
		
		
	}
	
	public void tearDown(){
		Mockit.tearDownMocks(Persistencia.class);
		Mockit.tearDownMocks(Calendar.class);
	}
	
	
	
	
	
	
	
	
	public void testCrearListaNocogen() {
		
		for(int i = 14;i<16;i++){
			c.borrarLista(2011, "Enero", -i);
		}
		c.borrarLista(2011, "Enero", -5);
		
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = -5;
		l[1] = 1;
		lista.add(l);
		boolean r = c.crearListas(2011, "Enero", lista);
		assertTrue(c.getMes("Enero", 2011).getListas().size()==0);
		assertTrue(r == false);
		
	}
	
	
	public void testAddCitaConFecha(){
		
		c.addCitaConFecha(-1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", -15);
		assertTrue(c.getMes("Enero", 2011).getListas().get(-15).getAsignados().containsKey(-1234));
	}
	
	
	
	
	public void testAddCItaSinFecha() {
		c.addCitaSinFecha(-1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero");
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().containsKey(-1234));
		
	}
	
	
	

	public void testAddCItaEspecial() {
		
		c.addCitaEspecial(-1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", -15);
		assertTrue(c.getMes("Enero", 2011).getListas().get(-15).getEspeciales().containsKey(-1234));
		
	}
	
	
	
	
	public void testAsignarCItaADa() {
		
		c.asignarDiaCita(-515, 2011, "Enero", -15);
		
		assertTrue(c.getMes("Enero", 2011).getListas().get(-15).getAsignados().containsKey(-515));
		assertTrue(!c.getMes("Enero", 2011).getSinAsignarDa().containsKey(-515));
	}
	
	
	
	public void testBorrarCitaNormalSinFecha(){
		
		c.borrarCita("Enero", 2011, -515, false, -1);
		assertTrue(!c.getMes("Enero", 2011).getSinAsignarDa().containsKey(-515));
	}
	
	
	
	public void testBorrarCitaNormalConFecha(){
		
		c.borrarCita("Enero", 2011, -115, false, -15);
		assertTrue(!c.getMes("Enero", 2011).getListas().get(-15).getAsignados().containsKey(-115));
	}
	
	public void testBorrarCitaEspecialConFecha(){
		
		c.borrarCita("Enero", 2011, -215, true, -15);
		assertTrue(!c.getMes("Enero", 2011).getListas().get(-15).getEspeciales().containsKey(-215));
	}

	
	
	public void testBuscarPorNH(){
		
		Vector<ResultadoBusqueda> res = c.buscarCitasPorNH(-315);
		assertTrue(res.size()==1);
		assertTrue(res.get(0).getC().getNombre().equals("test2"));
	}
	
	
	
	
	public void testCambiarListaDeDia(){


		c.cambiarListaDeDa(2011, "Enero", -15, -1);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(-1));
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(-15));
	}
	
	
	public void testCambiarListaDeDiaError(){

		boolean r = c.cambiarListaDeDa(2011, "Enero", -14, -15);
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
		
	}
	
	public void testCambiarMaxLista(){

		c.cambiarMaxLista(2011, "Enero", -15, -3);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(-15));
		assertTrue(c.getMes("Enero", 2011).getListas().get(-15).getCantidadMaxima()==-3);
	}
	
	

	public void testsetConf(){
		
		
		c.setConfiguration(-17);
		assertTrue(c.getMesesToStore()==-17);
	}
	
	public void testOrganizarAsignados(){
		
		
		c.oganizarAsignados(2011, "Enero");
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==0);
		
		
		
	}
	
	
	
	
	public void testAddCitaNeg2(){
		
		c.addCitaSinFecha(-1234, "pedro lopez", "no hay comentario", -1, 2011, "Enero");
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().containsKey(-1234));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().get(-1234).getPreferencia()==-1);
	}
	
	public void testAddCitaNeg3(){
		
	c.addCitaConFecha(-1234, "pedro lopez", "no hay comentario", -1, 2011, "Enero", -15);
	assertTrue(c.getMes("Enero", 2011).getListas().get(-15).getAsignados().containsKey(-1234));
	assertTrue(c.getMes("Enero", 2011).getListas().get(-15).getAsignados().get(-1234).getPreferencia()==-1);
	}
	
	public void testAddCitaNeg4(){
		
		c.addCitaSinFecha(1234, "pedro lopez", "no hay comentario", 0, 2012, "Enero");
		assertTrue(c.getMes("Enero", 2012).getAo()==2012);
		
	}
	
	public void testAddCitaNeg5(){
		
		c.addCitaSinFecha(1234, "pedro lopez", "no hay comentario", 0, -2012, "Enero");
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		
	}
	
	
	public void testAddCitaNeg6(){
		
		
		c.addCitaConFecha(1234, "pedro lopez", "no hay comentario", 0, -2012, "Enero",15);
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		
	}
	
	public void testAddCitaNeg7(){
		
		
		c.addCitaEspecial(1234, "pedro lopez", "no hay comentario", 0, -2012, "Enero",15);
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		
	}
	
	public void testAddCitaNeg8(){
		
		c.asignarDiaCita(1234, -2012, "Enero",15);
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		
	}
	
	public void testCrearListasNeg(){
		
		c.crearListas(-2012, "Enero", new Vector<Integer[]>());
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		
	}
	
	public void testCrearListasNeg2(){
		
		c.cambiarListaDeDa(-2012, "Enero", 10, 11);
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		assertTrue(v.errors.get(0).equals("Enero de -2012 no tiene listas creadas, por lo que no se pueden cambiar de da."));
	}
	
	public void testCrearListasNeg3(){
		
		c.cambiarMaxLista(-2012, "Enero", 10, 10);
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		assertTrue(v.errors.get(0).equals("Enero de -2012 no tiene listas creadas, por lo que no se pueden cambiar el mximo de citas."));
	}
	
	
	public void testCrearListasNeg4(){
		
		c.borrarLista(-2012, "Enero", 10);
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		assertTrue(v.errors.get(0).equals("Enero de -2012 no tiene listas creadas, por lo que no se pueden borrar."));
	}
	
	public void testBorrarLista(){
		
		Vector<Integer[]> v2 = new Vector<Integer[]>();
		Integer  [] d = new Integer[2];
		d[0] = 1;
		d[1] = 10;
		v2.add(d);
		c.crearListas(2012, "Enero", v2);
		
		c.borrarLista(2012, "Enero", -1);
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		assertTrue(v.errors.get(0).equals("No existe una lista el -1 de Enero de 2012"));
	}
	
	public void testBorrarLista2(){
		
		Vector<Integer[]> v2 = new Vector<Integer[]>();
		Integer  [] d = new Integer[2];
		d[0] = 1;
		d[1] = 10;
		v2.add(d);
		c.crearListas(-2012, "Enero", v2);
		
		c.borrarLista(-2012, "Enero", -1);
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		assertTrue(v.errors.get(0).equals("No existe una lista el -1 de Enero de -2012"));
	}
	
	public void testOrganizarAsignados2(){
		
		c.oganizarAsignados(-2012, "Enero");
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		
	}
	
	public void testBorrarCita(){
		
		c.borrarCita("Enero", -2012, 0, false, 1);
		
		assertTrue(c.getMes("Enero", -2012).getAo()==-2012);
		
	}
	
}
