package pruebas.mutacion.dominio;


import java.lang.reflect.Field;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.Vector;

import junit.framework.TestCase;

import mockit.Mockit;


import persistencia.Persistencia;
import pruebas.cobertura.MyCalendar;
import pruebas.cobertura.PersistenciaStubsInterface2;
import pruebas.cobertura.PersistenciaStubsInterfaceVacia;
import pruebas.cobertura.VentanaStub;
import dominio.Cita;
import dominio.Control;
import dominio.Mes;
import dominio.ResultadoBusqueda;
import dominio.exceptions.CitaConFechaNoAsignadaException;
import dominio.exceptions.CitaYaAsigndaException;

public class InitialTestsNeg2 extends TestCase{


	Control c;
	VentanaStub v;
	
	public void setUp(){
		
		
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterfaceVacia.class);
		Mockit.setUpMock(Calendar.class,MyCalendar.class);
		c = null;
		v = null;
		
		
		v = new VentanaStub();
		c = new Control(v);
		
		Hashtable<String, Mes> meses = new Hashtable<String, Mes>();
		
		Mes m = new Mes("Enero", -2011);
		for(int i = 14;i<16;i++){
			m.crearLista(-i, 10);
			try {
				m.asignarCitaConFecha(new Cita("test", -(100+i), "testcoment", 0), -i, false);
				m.asignarCitaConFecha(new Cita("tests", -(200+i), "testcomens", 0), -i, true);
				
				
				m.asignarCitaSinFecha(new Cita("test", -(500+i), "testcoment", 0));
				m.asignarCitaSinFecha(new Cita("test2", -(300+i), "testcoment2", -1));
				m.asignarCitaSinFecha(new Cita("test2", -(400+i), "testcoment3", -2));
			} catch (CitaConFechaNoAsignadaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (CitaYaAsigndaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		meses.put("-2011Enero", m);
		
		try {
			Field f = c.getClass().getDeclaredField("meses");
			f.setAccessible(true);
			f.set(c, meses);
		} catch (SecurityException e) {
			fail();
		} catch (NoSuchFieldException e) {
			fail();
		} catch (IllegalArgumentException e) {
			fail();
		} catch (IllegalAccessException e) {
			fail();
		}
		
		
	}
	
	public void tearDown(){
		Mockit.tearDownMocks(Persistencia.class);
		Mockit.tearDownMocks(Calendar.class);
	}
	
	
	public void testCrearLista() {
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = -15;
		l[1] = 15;
		lista.add(l);
		c.crearListas(-2012, "Junio", lista);
		assertTrue(c.getMes("Junio", -2012).getListas().containsKey(-15));
	}
	
	
	
	
	
	public void testCrearListaNocogen() {
		
		for(int i = 14;i<16;i++){
			c.borrarLista(-2011, "Enero", -i);
		}
		c.borrarLista(-2011, "Enero", -5);
		
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = -5;
		l[1] = 1;
		lista.add(l);
		boolean r = c.crearListas(-2011, "Enero", lista);
		assertTrue(c.getMes("Enero", -2011).getListas().size()==0);
		assertTrue(r == false);
		
	}
	
	
	public void testAddCitaConFecha(){
		
		c.addCitaConFecha(-1234, "pedro lopez", "no hay comentario", 0, -2011, "Enero", -15);
		assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getAsignados().containsKey(-1234));
	}
	
	
	
	public void testAddCItaSinFecha() {
		c.addCitaSinFecha(-1234, "pedro lopez", "no hay comentario", 0, -2011, "Enero");
		assertTrue(c.getMes("Enero", -2011).getSinAsignarDa().containsKey(-1234));
		
	}
	
	public void testAddCItaEspecial() {
		
		c.addCitaEspecial(-1234, "pedro lopez", "no hay comentario", 0, -2011, "Enero", -15);
		assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getEspeciales().containsKey(-1234));
		
	}
	
	
	public void testAsignarCItaADa() {
		
		c.asignarDiaCita(-515, -2011, "Enero", -15);
		
		assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getAsignados().containsKey(-515));
		assertTrue(!c.getMes("Enero", -2011).getSinAsignarDa().containsKey(-515));
	}
	
	
	
	public void testBorrarCitaNormalSinFecha(){
		
		c.borrarCita("Enero", -2011, -515, false, -1);
		assertTrue(!c.getMes("Enero", -2011).getSinAsignarDa().containsKey(-515));
	}
	
	
	
	public void testCambiarListaDeDia(){


		c.cambiarListaDeDa(-2011, "Enero", -15, -1);
		
		assertTrue(c.getMes("Enero", -2011).getListas().containsKey(-1));
		assertTrue(!c.getMes("Enero", -2011).getListas().containsKey(-15));
	}
	
	
	
	public void testCambiarMaxLista(){

		c.cambiarMaxLista(-2011, "Enero", -15, -3);
		
		assertTrue(c.getMes("Enero", -2011).getListas().containsKey(-15));
		assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getCantidadMaxima()==-3);
	}
	
	
	
	
	public void testOrganizarAsignados(){
		
		c.oganizarAsignados(-2011, "Enero");
		assertTrue(c.getMes("Enero", -2011).getSinAsignarDa().size()==0);
		
	}
	
	
	
	public void testAddCItaEspecial2() {
		
		c.addCitaEspecial(-1234, "pedro lopez", "no hay comentario", -1, -2011, "Enero", -15);
		assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getEspeciales().containsKey(-1234));
		assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getEspeciales().get(-1234).getPreferencia()==-1);
		
	}
	
}
