package pruebas.mutacion.dominio;


import java.util.Calendar;
import java.util.Hashtable;
import java.util.Vector;

import junit.framework.TestCase;

import mockit.Mockit;


import persistencia.Persistencia;
import pruebas.cobertura.MyCalendar;
import pruebas.cobertura.PersistenciaStubsInterface;
import pruebas.cobertura.VentanaStub;
import dominio.Cita;
import dominio.Control;
import dominio.Mes;
import dominio.ResultadoBusqueda;
import dominio.exceptions.CitaYaAsigndaException;

public class InitialTests_Mut extends TestCase{

		
	Control c;
	VentanaStub v;
	
	public void setUp(){
		
		
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface.class);
		Mockit.setUpMock(Calendar.class,MyCalendar.class);
		c = null;
		v = null;
		
		v = new VentanaStub();
		c = new Control(v);
		
		
	}
	
	public void tearDown(){
		Mockit.tearDownMocks(Persistencia.class);
		Mockit.tearDownMocks(Calendar.class);
	}
	
	
	
	
	
	/***********************ParaMutacion********************/
	
	public void testBuscarPorNombre2(){

		Vector<ResultadoBusqueda> res = c.buscarCitasPorNombre("test2");
		assertTrue(res.size()<10);
		assertTrue(res.get(0).getC().getNombre().contains("test2"));
		
	}
	
	public void testCrearLista2() {
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = 15;
		l[1] = 15;
		lista.add(l);
		c.crearListas(2011, "Enero", lista);
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getAsignados().size()+"",c.getMes("Enero", 2011).getListas().get(15).getAsignados().size()==4);
	}
	
	public void testBuscarPorNHNeg(){
		
		Vector<ResultadoBusqueda> res = c.buscarCitasPorNH(-115);
		assertTrue(res.size()==0);
		
	}
	
	public void testBuscarPorNHNeg2(){
		
		Vector<ResultadoBusqueda> res = c.buscarCitasPorNH(-215);
		assertTrue(res.size()==0);
		
	}
	
	public void testCambiarListaDeDiaError3(){

		boolean r = c.cambiarListaDeDa(2011, "Enero", -15, 2);
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
		
	}
	
	public void testAddCitaConFechaError(){
		
		c.addCitaConFecha(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", -15);
		assertTrue(!c.getMes("Enero", 2011).getListas().get(15).getAsignados().containsKey(1234));
	}
	
	public void testBorrarCitaEspecialSinFechaNeg(){
		Mes m = c.getMes("Enero", 2011);
		
		m.getSinAsignarDaEspecial().put(12345, new Cita("",12345,"",0));
		
		c.borrarCita("Enero", 2011, -12345, true, -1);
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDaEspecial().containsKey(12345));
	}
	
	
	public void testAsignarCItaADaEspNeg() {
		
		Mes m = c.getMes("Enero", 2011);
		
		m.getSinAsignarDaEspecial().put(-12345, new Cita("",-12345,"",0));
		c.asignarDiaCita(-12345, 2011, "Enero", 15);
		
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getEspeciales().containsKey(-12345));
		assertTrue(!c.getMes("Enero", 2011).getSinAsignarDaEspecial().containsKey(-12345));
	}
	
	
	/*public void testCambiarListaDeDia2(){


		c.cambiarListaDeDa(2011, "Enero", 15, 27);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(27));
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(15));
	}
	
	public void testCambiarListaDeDia3(){


		c.getMes("Enero", 2011).getListas().remove(15);
		
		c.cambiarListaDeDa(2011, "Enero", 14, 16);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(16));
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(14));
	}*/
	
	public void testCambiarListaDeDia4(){


		c.cambiarListaDeDa(2011, "Enero", 15, 2);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(2));
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(15));
		assertTrue(v.msg.get(0).equals("Cambio realizado sin problemas"));
	}
	
	
	public void testOrganizarAsignados2() throws CitaYaAsigndaException{
		Mes m = c.getMes("Enero", 2012);
		m.crearLista(5, 0);
		m.crearLista(18, 1);
		
		Cita ci = new Cita("", 123, "", Cita.APRINCIPIOS);
		m.asignarCitaSinFecha(ci);
		
		c.oganizarAsignados(2012, "Enero");
		assertTrue(c.getMes("Enero", 2012).getSinAsignarDa().size()==0);
		assertTrue(c.getMes("Enero", 2012).getListas().get(18).getAsignados().size()==1);
		
		
	}
	
	public void testOrganizarAsignados3() throws CitaYaAsigndaException{
		Mes m = c.getMes("Enero", 2012);
		m.crearLista(5, 1);
		m.crearLista(18, 0);
		
		Cita ci = new Cita("", 123, "", Cita.AFINALES);
		m.asignarCitaSinFecha(ci);
		
		c.oganizarAsignados(2012, "Enero");
		assertTrue(c.getMes("Enero", 2012).getSinAsignarDa().size()==0);
		assertTrue(c.getMes("Enero", 2012).getListas().get(5).getAsignados().size()==1);
		
		
	}
	
	public void testBuscarPorNombreEspeciales(){

		Mes m = c.getMes("Enero", 2011);
		m.getSinAsignarDaEspecial().put(123,new Cita("tests", 123, "", 0));
		Vector<ResultadoBusqueda> res = c.buscarCitasPorNombre("testsNinguno");
		assertTrue(res.size()==0);
		
		
	}
	
	public void testCambiarMaxLista2(){

		c.addCitaConFecha(1234, "", "", 0, 2011, "Enero", 15);
		c.cambiarMaxLista(2011, "Enero", 15, 0);
		
		assertTrue(c.getMes("Enero", 2011).getListas().containsKey(15));
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getCantidadMaxima()==0);
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getAsignados().size()==0);
		
	}
	
	public void testAddCItaSinFechaNoSeAade() {
		c.addCitaSinFecha(514, "pedro lopez", "no hay comentario", 0, 2011, "Enero");
		
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
		
	}
	
	public void testAddCItaEspecial() {
		c.addCitaEspecial(12345, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 14);
		
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
		
	}
	
	public void testAddCItaADia() {
		c.asignarDiaCita(514, 2011, "Enero", 14);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
		
	}
	public void testAddCItaADiaNoAsignada() {
		c.asignarDiaCita(5140, 2011, "Enero", 14);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
		
	}
	
	public void testCrearListas() {
		Vector<Integer[]> listas = new Vector<Integer[]>();
		Integer [] d = new Integer[2];
		d[0] = 1;
		d[1] = 2;
		listas.add(d);
		
		c.crearListas(2012, "Enero", listas);
		
		
		assertTrue(PersistenciaStubsInterface.database.get("2012Enero")!=null);
		
	}
	
	public void testCrearListasNoSeCrean() {
		Vector<Integer[]> listas = new Vector<Integer[]>();
		Integer [] d = new Integer[2];
		d[0] = 1;
		d[1] = 2;
		listas.add(d);
		
		for(int i = 0; i<4;i++){
			c.addCitaSinFecha(100+i, ""+i, ""+i, 0, 2012, "Enero");
		}
		
		PersistenciaStubsInterface.database = new Hashtable<String, Mes>();
		
		c.crearListas(2012, "Enero", listas);
		
		
		assertTrue(PersistenciaStubsInterface.database.get("2012Enero")==null);
		
	}
	
	
	public void testcambiarListaDeDia() {
		c.cambiarListaDeDa(2011, "Enero", 14, 4);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
		
	}
	
	public void testcambiarListaDeDiaNoCambia() {
		c.cambiarListaDeDa(2011, "Enero", 4, 14);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
		
	}
	
	public void testcambiarMaxLista() {
		c.cambiarMaxLista(2011, "Enero", 14, 20);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
		
	}
	
	public void testcambiarMaxListaNoCambia() {
		c.cambiarMaxLista(2011, "Enero", 4, 20);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
		
	}
	
	public void testBorrarLista(){
		c.borrarLista(2011, "Enero", 14);
		
		assertTrue(v.errors.size()==0);
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
	}
	
	public void testBorrarListaNoSeBorra(){
		c.borrarLista(2011, "Enero", 4);
		
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
	}
	
	
	public void testOrganizarAsignados(){
		c.oganizarAsignados(2011, "Enero");
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
	}
	
	public void testOrganizarAsignadosNoOrganizados(){
		c.addCitaSinFecha(12345, "", "", 0, 2011, "Febrero");
		
		PersistenciaStubsInterface.database = new Hashtable<String, Mes>();
		
		c.oganizarAsignados(2011, "Febrero");
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
	}
	
	public void testBorrarCita(){
		c.borrarCita("Enero", 2011, 514, false, -1);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")!=null);
	}
	
	public void testBorrarCitaNoBorra(){
		c.borrarCita("Enero", 2011, 5140, false, -1);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
	}
	
	
	public void testCrearListasNoSeCreanRaro() {
		for(int i = 0; i<40;i++){
			c.addCitaSinFecha(12300+i, ""+i, "", 0, 2011, "Enero");
		}
		
		Vector<Integer[]> listas = new Vector<Integer[]>();
		Integer [] d = new Integer[2];
		d[0] = 1;
		d[1] = 2;
		Integer [] d2 = new Integer[2];
		d2[0] = 14;
		d2[1] = 2;
		listas.add(d);
		listas.add(d2);
		
		PersistenciaStubsInterface.database = new Hashtable<String, Mes>();
		
		c.crearListas(2011, "Enero", listas);
		
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
		assertTrue(c.getMes("Enero", 2011).getListas().get(14).getNumeroTotal() == 2);
	}
	
	
	public void testAddCItaConFEchaCommet(){
		c.getMes("Agosto", 2012).crearLista(15, 10);
		c.addCitaConFecha(12345, "Pedro", "coment12345", 0, 2012, "Agosto", 15);
		
		Cita ci = c.getMes("Agosto", 2012).getListas().get(15).getAsignados().get(12345);
		assertTrue(ci.getComment().equals("coment12345"));
		assertTrue(ci.getNombre().equals("Pedro"));
		assertTrue(ci.getNumeroHistoria()==12345);
		assertTrue(ci.getPreferencia() == 0);
	}
	
	public void testAddCItaConFEchaCommentEspecial(){
		c.getMes("Agosto", 2012).crearLista(15, 10);
		c.addCitaEspecial(12345, "Pedro", "coment12345", 0, 2012, "Agosto", 15);
		
		Cita ci = c.getMes("Agosto", 2012).getListas().get(15).getEspeciales().get(12345);
		assertTrue(ci.getComment().equals("coment12345"));
		assertTrue(ci.getNombre().equals("Pedro"));
		assertTrue(ci.getNumeroHistoria()==12345);
		assertTrue(ci.getPreferencia() == 0);
	}
	
	
	public void testAddCitaConFechaDataBase(){
		
		c.addCitaConFecha(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", 15);
		assertTrue(c.getMes("Enero", 2011).getListas().get(15).getAsignados().containsKey(1234));
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero").getListas().get(15).getAsignados().containsKey(1234));
	}
	
	public void testAddCitaConFechaDataBaseError(){
		
		c.addCitaConFecha(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", -15);
		assertTrue(!c.getMes("Enero", 2011).getListas().get(15).getAsignados().containsKey(1234));
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
	}
	
	public void testAddCitaEspecialConFechaDataBaseError(){
		
		c.addCitaEspecial(1234, "pedro lopez", "no hay comentario", 0, 2011, "Enero", -15);
		assertTrue(!c.getMes("Enero", 2011).getListas().get(15).getEspeciales().containsKey(1234));
		assertTrue(PersistenciaStubsInterface.database.get("2011Enero")==null);
	}
}
