package pruebas.mutacion.dominio;


import java.util.Vector;

import dominio.Cita;
import dominio.Mes;
import dominio.ResultadoBusqueda;
import dominio.exceptions.CitaConFechaNoAsignadaException;
import dominio.exceptions.CitaNoExisteException;
import dominio.exceptions.CitaYaAsigndaException;
import dominio.exceptions.DemasiadasCitasException;
import dominio.exceptions.ListaNoCambiadaException;
import junit.framework.TestCase;

public class TestCubrirMes_Mut extends TestCase{

	
	Mes m;
	
	protected void setUp(){
		m=null;
		m = new Mes("Enero", 2011);
		
	}
	
	
	
	
	
	
	/*************Para mutacion***************/
	public void testSetGets2(){
		
		
		m.setAo(-2012);
		assertTrue(m.getAo()==-2012);
		
		m.setMes("Febrero");
		assertTrue(m.getMes().equals("Febrero"));
	}
	
	public void testBuscarCitaConnhNEgativo() throws CitaYaAsigndaException{
		
		Cita c = new Cita("p", -123, "c", 0);
		m.getSinAsignarDaEspecial().put(-123, c);
		
		Vector<ResultadoBusqueda> v = m.buscarCitaPorNH(-123);
		assertTrue(v.size()==1);
		assertTrue(v.get(0).getC()==c);
	}
	
	public void testCambiarDiaIniNEg() {
		
		
		try {
			m.cambiarListaDeDia(-1, 10);
			fail();
		} catch (ListaNoCambiadaException e) {
			assertTrue(e.getMessage().equals("No existe ninguna lista asigndad al dia -1"));
		}
		
		
	}
	
	public void testCambiarDiaFinNEg() {
		
		m.crearLista(-1, 10);
		m.crearLista(10, 10);
		try {
			m.cambiarListaDeDia(10, -1);
			fail();
		} catch (ListaNoCambiadaException e) {
			assertTrue(e.getMessage().equals("Ya hay una lista asigndad al dia -1"));
		}
		
		
	}
	
	public void testCambiarDiaFinNEg2() throws ListaNoCambiadaException {
		
		
		m.crearLista(10, 10);
		
		m.cambiarListaDeDia(10, -1);
			
		assertTrue(m.getListas().get(-1).getDia()==-1);
		
	}
	
	public void testCambiarDiaConCitasIniNeg() throws ListaNoCambiadaException, CitaConFechaNoAsignadaException, CitaYaAsigndaException {
		
		Cita c = new Cita("p", 123, "c", Cita.APRINCIPIOS);
		
		m.crearLista(-16, 10);
		m.asignarCitaConFecha(c, -16, false);
		
		String msg = m.cambiarListaDeDia(-16, 17);
		assertTrue(msg.equals("En la lista cambiada hay 1 personas que prefieren ser citadas a principios de mes, pero la lista se ha cambiado a la segunda quincena del mes.\nSera combeniente cambiar la cita de estas personas:\np\t123\n"));
		
		
	}
	
	public void testCambiarDiaConCitasFinNeg() throws ListaNoCambiadaException, CitaConFechaNoAsignadaException, CitaYaAsigndaException {
		
		Cita c = new Cita("p", 123, "c", Cita.APRINCIPIOS);
		
		m.crearLista(1, 10);
		m.asignarCitaConFecha(c, 1, false);
		
		String msg = m.cambiarListaDeDia(1, -17);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
		
	}
	
	public void testCambiarDiaConCitasIniNeg2() throws ListaNoCambiadaException, CitaConFechaNoAsignadaException, CitaYaAsigndaException {
		
		Cita c = new Cita("p", 123, "c", Cita.AFINALES);
		
		m.crearLista(-16, 10);
		m.asignarCitaConFecha(c, -16, false);
		
		String msg = m.cambiarListaDeDia(-16, 10);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
		
		
	}
	
	public void testCambiarDiaConCitasFinNeg2() throws ListaNoCambiadaException, CitaConFechaNoAsignadaException, CitaYaAsigndaException {
		
		Cita c = new Cita("p", 123, "c", Cita.AFINALES);
		
		m.crearLista(22, 10);
		m.asignarCitaConFecha(c, 22, false);
		
		String msg = m.cambiarListaDeDia(22, -20);
		assertTrue(msg.equals("En la lista cambiada hay 1 personas que prefieren ser citadas a finales de mes, pero la lista se ha cambiado a la primera quincena del mes.\nSera combeniente cambiar la cita de estas personas:\np\t123\n"));
		
		
	}
	
	public void testCambiarMaxDiaNeg(){
		
		try {
			m.cambiarMaxLista(-1, 5);
			fail();
		} catch (ListaNoCambiadaException e) {
			assertTrue(e.getMessage().equals("No existe ninguna lista asigndad al dia -1"));
		}
		
	}
	
	public void testAsignarCitaConFechaDiaNEgConListas() throws CitaYaAsigndaException{
		
		m.crearLista(1, 10);
		
		Cita c = new Cita("", 1, "", 0);
		
		try {
			m.asignarCitaConFecha(c, -1, false);
			fail();
		} catch (CitaConFechaNoAsignadaException e) {
			assertTrue(e.getMessage().equals("No hay ninguna lista asignda al -1 de Enero de 2011"));
		} 
		
	}

	public void testCambiarMaxNegConAsignados() throws ListaNoCambiadaException, CitaConFechaNoAsignadaException, CitaYaAsigndaException {
		
		Cita c = new Cita("p", 123, "c", Cita.AFINALES);
		Cita c2 = new Cita("p", 124, "c", Cita.AFINALES);
		
		m.crearLista(22, 10);
		m.asignarCitaConFecha(c, 22, false);
		m.asignarCitaConFecha(c2, 22, false);
		
		m.cambiarMaxLista(22, -1);
		
		assertTrue(m.getListas().get(22).getNumeroAsignados() == 0);
		assertTrue(m.getSinAsignarDa().size()==2);
	}
	
	public void testOrganizarAsignadosEspecial() throws CitaYaAsigndaException, DemasiadasCitasException  {
		
		Cita c = new Cita("p", 123, "c", Cita.APRINCIPIOS);
		Cita c2 = new Cita("p", 124, "c", Cita.AFINALES);
		
		m.crearLista(-22, 10);
		m.crearLista(22, 10);
		m.asignarCitaSinFecha(c);
		m.asignarCitaSinFecha(c2);
		
		m.organizarAsignados();
		
		assertTrue(m.getListas().get(22).getNumeroAsignados() == 1);
		assertTrue(m.getListas().get(-22).getNumeroAsignados() == 1);
		assertTrue(m.getListas().get(22).getAsignados().get(124)==c2);
		assertTrue(m.getListas().get(-22).getAsignados().get(123)==c);
	}
	
	public void testOrganizarAsignadosEspecial2() throws CitaYaAsigndaException, DemasiadasCitasException  {
		
		Cita c = new Cita("p", 123, "c", Cita.APRINCIPIOS);
		
		
		m.crearLista(-22, 10);
		
		m.asignarCitaSinFecha(c);
		
		
		m.organizarAsignados();
		
		
		assertTrue(m.getListas().get(-22).getNumeroAsignados() == 1);
		
		assertTrue(m.getListas().get(-22).getAsignados().get(123)==c);
	}
	
	public void testOrganizarAsignadosEspecial3() throws CitaYaAsigndaException, DemasiadasCitasException  {
		
		Cita c = new Cita("p", 123, "c", Cita.APRINCIPIOS);
		Cita c2 = new Cita("p", 124, "c", Cita.APRINCIPIOS);
		
		m.crearLista(-22, 10);
		
		m.asignarCitaSinFecha(c);
		m.asignarCitaSinFecha(c2);
		
		m.organizarAsignados();
		
		
		assertTrue(m.getListas().get(-22).getNumeroAsignados() == 2);
		
		assertTrue(m.getListas().get(-22).getAsignados().get(123)==c);
		assertTrue(m.getListas().get(-22).getAsignados().get(124)==c2);
	}
	
	public void testOrganizarAsignadosEspecial4() throws CitaYaAsigndaException, DemasiadasCitasException  {
		
		Cita c = new Cita("p", 123, "c", Cita.DAIGUAL);
		Cita c2 = new Cita("p", 124, "c", Cita.DAIGUAL);
		Cita c3 = new Cita("p", 125, "c", Cita.DAIGUAL);
		
		m.crearLista(-22, 2);
		m.crearLista(-21, 1);
		
		m.asignarCitaSinFecha(c);
		m.asignarCitaSinFecha(c2);
		m.asignarCitaSinFecha(c3);
		
		m.organizarAsignados();
		
		
		assertTrue(m.getListas().get(-22).getNumeroAsignados() == 2);
		assertTrue(m.getListas().get(-21).getNumeroAsignados() == 1);
		
		assertTrue(m.getListas().get(-22).getAsignados().get(124)==c2);
		assertTrue(m.getListas().get(-22).getAsignados().get(125)==c3);
		assertTrue(m.getListas().get(-21).getAsignados().get(123)==c);
	}
	
	public void testAsignarCitaADiaNHneg() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException{
		
		try {
			m.asignarDaACitaSinDa(-1, 10);
			fail();
		} catch (CitaNoExisteException e) {
			assertTrue(e.getMessage().equals("La persona con el nmero de historia -1 no se encuentra citada sin fecha"));
		} 
		
		
	}
	
	public void testAsignarCitaADiaDiaNeg() throws CitaYaAsigndaException, CitaNoExisteException {
		
		Cita c = new Cita("pedro", 123, "c", 0);
		
		m.asignarCitaSinFecha(c);
		
		try {
			m.asignarDaACitaSinDa(123, -1);
			fail();
		}  catch (CitaConFechaNoAsignadaException e) {
			assertTrue(e.getMessage().equals("No hay ninguna lista asignda al -1 de Enero de 2011"));
		}
		
	}
	
	
	public void testCambiarListaDeDia() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(16, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.APRINCIPIOS);
		m.asignarCitaConFecha(c, 16, false);
		
		String msg = m.cambiarListaDeDia(16, 17);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia2() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(15, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.APRINCIPIOS);
		m.asignarCitaConFecha(c, 15, false);
		
		String msg = m.cambiarListaDeDia(15, 17);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia3() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(14, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.APRINCIPIOS);
		m.asignarCitaConFecha(c, 14, false);
		
		String msg = m.cambiarListaDeDia(14, 15);
		assertTrue(!msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia4() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(14, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.AFINALES);
		m.asignarCitaConFecha(c, 14, false);
		
		String msg = m.cambiarListaDeDia(14, 16);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia5() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(14, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.AFINALES);
		m.asignarCitaConFecha(c, 14, true);
		
		String msg = m.cambiarListaDeDia(14, 16);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia6() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(15, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.AFINALES);
		m.asignarCitaConFecha(c, 15, false);
		
		String msg = m.cambiarListaDeDia(15, 14);
		assertTrue(!msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia7() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(15, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.AFINALES);
		m.asignarCitaConFecha(c, 15, false);
		
		String msg = m.cambiarListaDeDia(15, 16);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia8() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(16, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.AFINALES);
		m.asignarCitaConFecha(c, 16, false);
		
		String msg = m.cambiarListaDeDia(16, 15);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia9() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(16, 10);
		Cita c = new Cita("pedro", 123, "c", 3);
		m.asignarCitaConFecha(c, 16, false);
		
		String msg = m.cambiarListaDeDia(16, 14);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testCambiarListaDeDia10() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException, ListaNoCambiadaException{
		
		m.crearLista(16, 10);
		Cita c = new Cita("pedro", 123, "c", 3);
		m.asignarCitaConFecha(c, 16, true);
		
		String msg = m.cambiarListaDeDia(16, 14);
		assertTrue(msg.equals("Cambio realizado sin problemas"));
	}
	
	public void testOrganizarAsignadosPreferenciasMAlas() throws CitaYaAsigndaException, DemasiadasCitasException {
		
		m.crearLista(16, 10);
		m.crearLista(14, 10);
		Cita c = new Cita("pedro", 123, "c", -1);
		m.asignarCitaSinFecha(c);
		
		m.organizarAsignados();
		
		assertTrue(m.getListas().get(16).getAsignados().size()==1);
		assertTrue(m.getListas().get(14).getAsignados().size()==0);
	
	}
	
	
	public void testBorrarCitaDobleAsignada() throws CitaConFechaNoAsignadaException, CitaYaAsigndaException {
		
		m.crearLista(16, 10);
		m.crearLista(14, 10);
		Cita c = new Cita("pedro", 123, "c", -1);
		m.asignarCitaConFecha(c, 14, false);
		m.asignarCitaConFecha(c, 16, false);
		
		m.borrarCita(123, false, 14);
		
		assertTrue(m.getListas().get(16).getAsignados().size()==1);
		assertTrue(m.getListas().get(14).getAsignados().size()==0);
	
	}
	
	public void testOrganizarAsignadosNoseComo() throws CitaYaAsigndaException, DemasiadasCitasException {
		
		m.crearLista(16, 10);
		m.crearLista(14, 10);
		Cita c = new Cita("pedro", 123, "c", Cita.DAIGUAL);
		Cita c2 = new Cita("pedro", 124, "c", Cita.DAIGUAL);
		m.asignarCitaSinFecha(c);
		m.asignarCitaSinFecha(c2);
		
		String msg = m.organizarAsignados();
		assertTrue(msg.equals("Citas organizadas correctamente"));
		assertTrue(m.getListas().get(16).getAsignados().size()==1);
		assertTrue(m.getListas().get(14).getAsignados().size()==1);
	
	}
	
	public void testOrganizarAsignadosNoseComo2() throws CitaYaAsigndaException, DemasiadasCitasException {
		
		m.crearLista(16, 10);
		m.crearLista(14, 10);
		Cita c = new Cita("pedro", 123, "c", 3);
		Cita c2 = new Cita("pedro", 124, "c", 3);
		m.asignarCitaSinFecha(c);
		m.asignarCitaSinFecha(c2);
		
		String msg = m.organizarAsignados();
		assertTrue(msg.equals("Citas organizadas correctamente"));
		assertTrue(m.getListas().get(16).getAsignados().size()==1);
		assertTrue(m.getListas().get(14).getAsignados().size()==1);
	
	}
	
	public void testOrganizarAsignadosNoseComo3() throws CitaYaAsigndaException, DemasiadasCitasException {
		
		m.crearLista(16, 1);
		m.crearLista(14, 1);
		Cita c = new Cita("pedro", 123, "c", Cita.AFINALES);
		Cita c2 = new Cita("pedro", 124, "c", Cita.AFINALES);
		m.asignarCitaSinFecha(c);
		m.asignarCitaSinFecha(c2);
		
		String msg = m.organizarAsignados();
		assertTrue(!msg.equals("Citas organizadas correctamente"));
		assertTrue(m.getListas().get(16).getAsignados().size()==1);
		assertTrue(m.getListas().get(14).getAsignados().size()==1);
	
	}
	
	public void testOrganizarAsignadosNoCogen() throws CitaYaAsigndaException  {
		
		m.crearLista(16, 1);
		m.crearLista(14, 1);
		Cita c = new Cita("pedro", 123, "c", Cita.AFINALES);
		Cita c2 = new Cita("pedro", 124, "c", Cita.AFINALES);
		Cita c3 = new Cita("pedro", 125, "c", Cita.AFINALES);
		m.asignarCitaSinFecha(c);
		m.asignarCitaSinFecha(c2);
		m.asignarCitaSinFecha(c3);
		
		try {
			m.organizarAsignados();
			fail();
		} catch (DemasiadasCitasException e) {
			assertTrue(e.getMessage().equals("El nmero de personas citadas este mes excede el mximo de huecos en las listas creadas. Intente crear ms listas o aumentar el tamao mximo de alguna lista.\nTotal personas citadas: 3\nTotal huecos disponibles: 2"));
		}
		
		assertTrue(m.getListas().get(16).getAsignados().size()==0);
		assertTrue(m.getListas().get(14).getAsignados().size()==0);
	
	}
}
