package pruebas.mutacion.presentacion;

import mockit.Mockit;

import org.uispec4j.Button;
import org.uispec4j.Panel;
import org.uispec4j.Table;
import org.uispec4j.TextBox;
import org.uispec4j.Trigger;
import org.uispec4j.UISpecTestCase;
import org.uispec4j.Window;
import org.uispec4j.interception.WindowHandler;
import org.uispec4j.interception.WindowInterceptor;

import dominio.Control;

import persistencia.Persistencia;
import presentacion.JDBuscarCitas;
import pruebas.cobertura.PersistenciaStubsInterface;
import pruebas.cobertura.VentanaStub;


public class TestJDBuscarCIta_Mut extends UISpecTestCase{

	public void setUp(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface.class);
		//setAdapter(new MainClassAdapter(CitasMEd.class, new String[0]));
		
		
	}
	
	public void tearDown(){
		Mockit.tearDownMocks(Persistencia.class);
		
	}
	
	
	
	public void testBuscarCitaPorNHNeg(){
		
		VentanaStub vs = new VentanaStub();
		Control c = new Control(vs);
		c.addCitaSinFecha(-123, "pedro", "comentario", 0, 2011, "Enero");
		
		JDBuscarCitas jdbc = new JDBuscarCitas(null, c);
		Panel p = new Panel(jdbc);
		
		p.getTextBox("fieldNH").setText("-123");
		Button b1 = p.getButton("Buscar por nmero de historia");
		b1.click();
		
		Table t = p.getTable();
		assertTrue(t.rowCountEquals(1));
		
		t.selectRowsWithText(2,"-123");
		b1 = p.getButton("Borrar cita seleccionada");
		b1.click();
		
		t = p.getTable();
		assertTrue(t.getRowCount() == 0);
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(-123));
		
		
	}
	
	public void testBuscarCitaPorNombre(){
		
		VentanaStub vs = new VentanaStub();
		Control c = new Control(vs);
		
		JDBuscarCitas jdbc = new JDBuscarCitas(null, c);
		Panel p = new Panel(jdbc);
		
		p.getTextBox("fieldN").setText("test");
		
		Button b1 = p.getButton("Buscar por nombre");
		b1.click();
		
		Table t = p.getTable();
		assertTrue(t.rowCountEquals(10));
		
		b1 = p.getButton("Borrar cita seleccionada");
		
		try{
			b1.click();
			fail();
		}catch(Exception e){
			
		}
		
		assertTrue(t.getRowCount() == 10);
	}
	
	
	public void testBorrarCitaAoNeg(){
		
		VentanaStub vs = new VentanaStub();
		Control c = new Control(vs);
		c.addCitaSinFecha(12345, "pedro", "comentario", 0, -2011, "Enero");
		assertTrue(c.getMes("Enero", -2011).getSinAsignarDa().containsKey(12345));
		
		JDBuscarCitas jdbc = new JDBuscarCitas(null, c);
		Panel p = new Panel(jdbc);
		
		p.getTextBox("fieldNH").setText("12345");
		Button b1 = p.getButton("Buscar por nmero de historia");
		b1.click();
		
		Table t = p.getTable();
		assertTrue(t.rowCountEquals(1));
		
		t.selectRowsWithText(2,"12345");
		b1 = p.getButton("Borrar cita seleccionada");
		b1.click();
		
		t = p.getTable();
		assertTrue(t.getRowCount() == 0);
		assertTrue(!c.getMes("Enero", -2011).getSinAsignarDa().containsKey(12345));
		
		
	}
	
	public void testBorrarCitaNoSeBorra(){
		
		VentanaStub vs = new VentanaStub();
		Control c = new Control(vs);
		c.addCitaSinFecha(-123, "pedro", "comentario", 0, 2011, "Enero");
		
		JDBuscarCitas jdbc = new JDBuscarCitas(null, c);
		Panel p = new Panel(jdbc);
		
		p.getTextBox("fieldNH").setText("-123");
		Button b1 = p.getButton("Buscar por nmero de historia");
		b1.click();
		
		Table t = p.getTable();
		assertTrue(t.rowCountEquals(1));
		
		t.selectRowsWithText(2,"-123");
		
		
		boolean borrada = c.borrarCita("Enero", 2011, -123, false, -1);
		assertTrue(borrada==true);
		
		b1 = p.getButton("Borrar cita seleccionada");
		b1.click();
		
		t = p.getTable();
		assertTrue(t.getRowCount() == 1);
		
		
		
	}
	
	
	public void testBorrarCitaAvanzada(){
		
		VentanaStub vs = new VentanaStub();
		Control c = new Control(vs);
		
		JDBuscarCitas jdbc = new JDBuscarCitas(null, c);
		Panel p = new Panel(jdbc);
		
		p.getTextBox("fieldN").setText("test");
		
		Button b1 = p.getButton("Buscar por nombre");
		b1.click();
		
		Table t = p.getTable();
		assertTrue(t.rowCountEquals(10));
		
		t.selectRow(5);
		
		b1 = p.getButton("Borrar cita seleccionada");
		b1.click();
			
		
		assertTrue(t.getRowCount() == 9);
	}
	
	public void testBuscarCitaPorNH(){
		
		VentanaStub vs = new VentanaStub();
		Control c = new Control(vs);
		c.addCitaSinFecha(123, "pedro", "comentario", 3, 2011, "Enero");
		
		JDBuscarCitas jdbc = new JDBuscarCitas(null, c);
		Panel p = new Panel(jdbc);
		
		p.getTextBox("fieldNH").setText("123");
		Button b1 = p.getButton("Buscar por nmero de historia");
		b1.click();
		
		Table t = p.getTable();
		assertTrue(t.rowCountEquals(1));
		t.selectRow(0);
		
		assertTrue(p.getTextBox("fieldO").getText().equals("\ncomentario"));
		
	}
}
