package pruebas.sobran.dominio;


import java.util.Calendar;
import java.util.Vector;

import junit.framework.TestCase;

import mockit.Mockit;


import persistencia.Persistencia;
import pruebas.cobertura.MyCalendar;
import pruebas.cobertura.PersistenciaStubsInterface2;
import pruebas.cobertura.PersistenciaStubsInterfaceNeg;
import pruebas.cobertura.VentanaStub;
import dominio.Control;
import dominio.Mes;
import dominio.ResultadoBusqueda;

public class InitialTestsNeg extends TestCase{


	
	Control c;
	VentanaStub v;
	
	public void setUp(){
		
		
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterfaceNeg.class);
		Mockit.setUpMock(Calendar.class,MyCalendar.class);
		c= null;
		v = null;
		
		v = new VentanaStub();
		c = new Control(v);
		
		
	}
	
	public void tearDown(){
		Mockit.tearDownMocks(Persistencia.class);
		Mockit.tearDownMocks(Calendar.class);
	}
	
	
	public void testCrearLista() {
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = -15;
		l[1] = 15;
		lista.add(l);
		c.crearListas(2012, "Junio", lista);
		assertTrue(c.getMes("Junio", 2012).getListas().containsKey(-15));
	}
	
	public void testCrearListaErrorDB() {
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		Vector<Integer[]> lista = new Vector<Integer[]>();
		Integer [] l = new Integer[2];
		l[0] = -15;
		l[1] = 15;
		lista.add(l);
		c.crearListas(2012, "Junio", lista);
		assertTrue(c.getMes("Junio", 2012).getListas().containsKey(-15));
	}
	
	
	
	public void testAddCitaConFechaMesVacio(){	
		c.addCitaConFecha(-1234, "pedro lopez", "no hay comentario", 0, 2012, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaConFechaYaAsignada(){
		
		c.addCitaConFecha(-115, "pedro lopez", "no hay comentario", 0, 2011, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	
	public void testAddCItaSinFechaEnMesVacio() {
		c.addCitaSinFecha(-1234, "pedro lopez", "no hay comentario", 0, 2012, "Enero");
		assertTrue(c.getMes("Enero", 2012).getSinAsignarDa().containsKey(-1234));
		
	}
	
	public void testAddCItaSinFechaYaAsignada() {
		c.addCitaSinFecha(-515, "test", "no hay comentario", 0, 2011, "Enero");
		assertTrue(v.errors.size()==1);
		
	}
	
	
	public void testAddCitaEspecialConFechaMesVacio(){	
		
		c.addCitaEspecial(-1234, "pedro lopez", "no hay comentario", 0, 2012, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaEspecialConFechaYaAsignada(){
		
		c.addCitaEspecial(-215, "pedro lopez", "no hay comentario", 0, 2011, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	
	
	
	public void testAsignarCItaADaMesVacio(){	
		c.asignarDiaCita(-515, 2012, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaYaAsignada(){
		
		
		c.addCitaSinFecha(-115, "", "", 0, 2011, "Enero");
		c.asignarDiaCita(-115, 2011, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaSinLista(){

		c.asignarDiaCita(-515, 2011, "Enero", -2);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaErrorGuardarDataBase(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.asignarDiaCita(-515, 2011, "Enero", -15);
		assertTrue(v.errors.size()==0);
	}
	
	
	public void testBorrarCitaNormalSinFechaMesVacio(){
		
		boolean r = c.borrarCita("Enero", 2012, -515, false, -1);
		assertTrue(r == false);
	}
	

	public void testBorrarLista(){
		
		c.borrarLista(2011, "Enero", -15);
		
		assertTrue(!c.getMes("Enero", 2011).getListas().containsKey(-15));
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDa().size()==7);
		assertTrue(c.getMes("Enero", 2011).getSinAsignarDaEspecial().size()==1);
	}
	
	public void testBorrarListaMesVacio(){
		
		boolean r = c.borrarLista(2012, "Enero", -15);
		
		assertTrue(v.errors.size()==1);
		assertTrue(r == false);
		
	}
	
	
	
	public void testCambiarListaDeDiaMesVacio(){


		c.cambiarListaDeDa(2012, "Enero", -15, -1);
		assertTrue(v.errors.size()==1);
		
	}
	
	
	public void testCambiarMaxListaMesVacio(){

		c.cambiarMaxLista(2012, "Enero", -15, -3);
		assertTrue(v.errors.size()==1);
	}
	
	
	public void testCambiarMaxListaError(){

		boolean r = c.cambiarMaxLista(2011, "Enero", -1, -3);
		
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
	}
	
	public void testGetMes(){
		
		
		
		Mes m = c.getMes("Enero", 2011);
		assertTrue(m.getListas().containsKey(-15));
		assertTrue(m.getListas().get(-15).getCantidadMaxima()==10);
		
		Mes m2 = c.getMes("Febrero", 2011);
		assertTrue(!m2.getListas().containsKey(-15));
		
		
	}

	
	public void testGetMesesToStore(){
		
		int me = c.getMesesToStore();
		assertTrue(me == 24);
		
	}
	public void testGetPathCitas(){
		
		String me = c.getPathDBCitas();
		assertTrue(me.equals("."));
		
	}
	
	
}
