package pruebas.sobran.dominio;


import java.lang.reflect.Field;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.Vector;

import junit.framework.TestCase;

import mockit.Mockit;


import persistencia.Persistencia;
import pruebas.cobertura.MyCalendar;
import pruebas.cobertura.PersistenciaStubsInterface2;
import pruebas.cobertura.PersistenciaStubsInterfaceVacia;
import pruebas.cobertura.VentanaStub;
import dominio.Cita;
import dominio.Control;
import dominio.Mes;
import dominio.ResultadoBusqueda;
import dominio.exceptions.CitaConFechaNoAsignadaException;
import dominio.exceptions.CitaYaAsigndaException;

public class InitialTestsNeg2 extends TestCase{


	Control c;
	VentanaStub v;
	
	public void setUp(){
		
		
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterfaceVacia.class);
		Mockit.setUpMock(Calendar.class,MyCalendar.class);
		c= null;
		v = null;
		
		v = new VentanaStub();
		c = new Control(v);
		
		Hashtable<String, Mes> meses = new Hashtable<String, Mes>();
		
		Mes m = new Mes("Enero", -2011);
		for(int i = 14;i<16;i++){
			m.crearLista(-i, 10);
			try {
				m.asignarCitaConFecha(new Cita("test", -(100+i), "testcoment", 0), -i, false);
				m.asignarCitaConFecha(new Cita("tests", -(200+i), "testcomens", 0), -i, true);
				
				
				m.asignarCitaSinFecha(new Cita("test", -(500+i), "testcoment", 0));
				m.asignarCitaSinFecha(new Cita("test2", -(300+i), "testcoment2", -1));
				m.asignarCitaSinFecha(new Cita("test2", -(400+i), "testcoment3", -2));
			} catch (CitaConFechaNoAsignadaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (CitaYaAsigndaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		meses.put("-2011Enero", m);
		
		try {
			Field f = c.getClass().getDeclaredField("meses");
			f.setAccessible(true);
			f.set(c, meses);
		} catch (SecurityException e) {
			fail();
		} catch (NoSuchFieldException e) {
			fail();
		} catch (IllegalArgumentException e) {
			fail();
		} catch (IllegalAccessException e) {
			fail();
		}
		
		
	}
	
	public void tearDown(){
		Mockit.tearDownMocks(Persistencia.class);
		Mockit.tearDownMocks(Calendar.class);
	}
	
	
	
	public void testAddCitaConFechaMesVacio(){	
		c.addCitaConFecha(-1234, "pedro lopez", "no hay comentario", 0, -2012, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaConFechaYaAsignada(){
		
		c.addCitaConFecha(-115, "pedro lopez", "no hay comentario", 0, -2011, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	
	public void testAddCItaSinFechaEnMesVacio() {
		c.addCitaSinFecha(-1234, "pedro lopez", "no hay comentario", 0, -2012, "Enero");
		assertTrue(c.getMes("Enero", -2012).getSinAsignarDa().containsKey(-1234));
		
	}
	
	public void testAddCItaSinFechaYaAsignada() {
		c.addCitaSinFecha(-515, "test", "no hay comentario", 0, -2011, "Enero");
		assertTrue(v.errors.size()==1);
		
	}
	
	
	public void testAddCitaEspecialConFechaMesVacio(){	
		
		c.addCitaEspecial(-1234, "pedro lopez", "no hay comentario", 0, -2012, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAddCitaEspecialConFechaYaAsignada(){
		
		c.addCitaEspecial(-215, "pedro lopez", "no hay comentario", 0, -2011, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	
	
	public void testAsignarCItaADaMesVacio(){	
		c.asignarDiaCita(-515, -2012, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaYaAsignada(){
		
		
		c.addCitaSinFecha(-115, "", "", 0, -2011, "Enero");
		c.asignarDiaCita(-115, -2011, "Enero", -15);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaSinLista(){

		c.asignarDiaCita(-515, -2011, "Enero", -2);
		assertTrue(v.errors.size()==1);
	}
	
	public void testAsignarCItaADaErrorGuardarDataBase(){
		Mockit.setUpMock(Persistencia.class,PersistenciaStubsInterface2.class);
		c.asignarDiaCita(-515, -2011, "Enero", -15);
		assertTrue(v.errors.size()==0);
	}
	
	
	public void testBorrarCitaNormalSinFechaMesVacio(){
		
		boolean r = c.borrarCita("Enero", -2012, -515, false, -1);
		assertTrue(r == false);
	}
	
	
	public void testBorrarCitaNormalConFecha(){
		
		c.borrarCita("Enero", -2011, -115, false, -15);
		assertTrue(!c.getMes("Enero", -2011).getListas().get(-15).getAsignados().containsKey(-115));
	}
	
	public void testBorrarCitaEspecialConFecha(){
		
		c.borrarCita("Enero", -2011, -215, true, -15);
		assertTrue(!c.getMes("Enero", -2011).getListas().get(-15).getEspeciales().containsKey(-215));
	}

	public void testBorrarLista(){
		
		c.borrarLista(-2011, "Enero", -15);
		
		assertTrue(!c.getMes("Enero", -2011).getListas().containsKey(-15));
		assertTrue(c.getMes("Enero",-2011).getSinAsignarDa().size()==7);
		assertTrue(c.getMes("Enero", -2011).getSinAsignarDaEspecial().size()==1);
	}
	
	public void testBorrarListaMesVacio(){
		
		boolean r = c.borrarLista(-2012, "Enero", -15);
		
		assertTrue(v.errors.size()==1);
		assertTrue(r == false);
		
	}
	
	
	
	
	public void testBuscarPorNH(){
		
		Vector<ResultadoBusqueda> res = c.buscarCitasPorNH(-315);
		assertTrue(res.size()==1);
		assertTrue(res.get(0).getC().getNombre().equals("test2"));
	}
	
	
	
	public void testCambiarListaDeDiaMesVacio(){


		c.cambiarListaDeDa(-2012, "Enero", -15, -1);
		assertTrue(v.errors.size()==1);
		
	}
	
	
	
	public void testCambiarListaDeDiaError(){

		boolean r = c.cambiarListaDeDa(-2011, "Enero", -14, -15);
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
		
	}
	
	
	public void testCambiarMaxListaMesVacio(){

		c.cambiarMaxLista(-2012, "Enero", -15, -3);
		assertTrue(v.errors.size()==1);
	}
	
	
	public void testCambiarMaxListaError(){

		boolean r = c.cambiarMaxLista(-2011, "Enero", -1, -3);
		
		assertTrue(r == false);
		assertTrue(v.errors.size()==1);
	}
	
	public void testGetMes(){
		
		
		
		Mes m = c.getMes("Enero", -2011);
		assertTrue(m.getListas().containsKey(-15));
		assertTrue(m.getListas().get(-15).getCantidadMaxima()==10);
		
		Mes m2 = c.getMes("Febrero", -2011);
		assertTrue(!m2.getListas().containsKey(-15));
		
		
	}

	
	
	
	
	
	
	
	public void testAddCitaNeg2(){
		
		c.addCitaSinFecha(-1234, "pedro lopez", "no hay comentario", -1, -2011, "Enero");
		assertTrue(c.getMes("Enero", -2011).getSinAsignarDa().containsKey(-1234));
		assertTrue(c.getMes("Enero", -2011).getSinAsignarDa().get(-1234).getPreferencia()==-1);
	}
	
	public void testAddCitaNeg3(){
		
	c.addCitaConFecha(-1234, "pedro lopez", "no hay comentario", -1, -2011, "Enero", -15);
	assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getAsignados().containsKey(-1234));
	assertTrue(c.getMes("Enero", -2011).getListas().get(-15).getAsignados().get(-1234).getPreferencia()==-1);
	}
	
	
	public void testAsignarCItaADaMesVacio2(){	
		c.addCitaSinFecha(12354657, "n", "c", -2, 2012, "Enero");
		c.getMes("Enero", 2012).crearLista(-5, 10);
		c.asignarDiaCita(-505, -2012, "Enero", -5);
		assertTrue(v.errors.size()==1);
	}
	
	public void testCambiarListaDeDiaMesVacio2(){

		
		c.getMes("Enero", 2012).crearLista(-15, 10);
		c.cambiarListaDeDa(-2012, "Enero", -15, -1);
		assertTrue(v.errors.size()==1);
		
	}
	
	
	public void testCambiarMaxListaMesVacio2(){
		
		c.getMes("Enero", 2012).crearLista(-15, 10);
		c.cambiarMaxLista(-2012, "Enero", -15, -3);
		assertTrue(v.errors.size()==1);
	}
	
	public void testBorrarListaMesVacio2(){
		
		c.getMes("Enero", 2012).crearLista(-15, 10);
		boolean r = c.borrarLista(-2012, "Enero", -15);
		
		assertTrue(v.errors.size()==1);
		assertTrue(r == false);
		
	}
	
	public void testOrganizarAsignadosMesVacio2(){
		
		c.getMes("Enero", 2012).crearLista(-15, 1);
		c.addCitaSinFecha(1234, "", "", 0, 2012, "Enero");
		c.addCitaSinFecha(12345, "", "", 0, 2012, "Enero");
		boolean r = c.oganizarAsignados(-2012, "Enero");
		assertTrue(r == true);
		
		
		
	}
	
}
